from plone.app.layout.links.viewlets import FaviconViewlet
from plone.app.layout.testing import FUNCTIONAL_TESTING
from plone.app.layout.viewlets.tests.base import ViewletsTestCase
from plone.base.interfaces import ISiteSchema
from plone.formwidget.namedfile.converter import b64encode_file
from plone.registry.interfaces import IRegistry
from zope.component import getUtility


class TestFaviconViewletView(ViewletsTestCase, FaviconViewlet):
    layer = FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = ""
        self.site_url = ""

    def test_FaviconViewlet_get_mimetype_svg(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone")
        filename = "test.svg"
        file_data = b"Hello World"
        encoded_data = b64encode_file(filename=filename, data=file_data)
        settings.site_favicon = encoded_data
        mimetype = settings.site_favicon_mimetype
        self.assertEqual(mimetype, "image/svg+xml")

    def test_FaviconViewlet_get_mimetype_jpg(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone")
        filename = "test.jpg"
        file_data = b"Hello World"
        encoded_data = b64encode_file(filename=filename, data=file_data)
        settings.site_favicon = encoded_data
        mimetype = settings.site_favicon_mimetype
        self.assertEqual(mimetype, "image/jpeg")

    def test_FaviconViewlet_get_mimetype_png(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone")
        filename = "test.png"
        file_data = b"Hello World"
        encoded_data = b64encode_file(filename=filename, data=file_data)
        settings.site_favicon = encoded_data
        mimetype = settings.site_favicon_mimetype
        self.assertEqual(mimetype, "image/png")

    def test_FaviconViewlet_get_mimetype_ico(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone")
        filename = "test.ico"
        file_data = b"Hello World"
        encoded_data = b64encode_file(filename=filename, data=file_data)
        settings.site_favicon = encoded_data
        mimetype = settings.site_favicon_mimetype
        self.assertEqual(mimetype, "image/vnd.microsoft.icon")

    def test_FaviconViewlet_get_mimetype_none(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone")
        settings.site_favicon = None
        mimetype = settings.site_favicon_mimetype
        self.assertEqual(mimetype, "image/vnd.microsoft.icon")



from plone.app.layout.viewlets.content import ContentHistoryViewlet
from plone.app.layout.viewlets.content import WorkflowHistoryViewlet
from plone.app.layout.viewlets.tests.base import ViewletsFunctionalTestCase
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD


class TestWorkflowHistoryViewlet(ViewletsFunctionalTestCase):
    def setUp(self):
        super().setUp()
        # add document, perform transition, set history for non-existent
        # member and also None (anonymous)
        self.folder.invokeFactory("Document", "d1")

    def addMember(self, username, roles=("Member",)):
        self.portal.portal_membership.addMember(username, TEST_USER_PASSWORD, roles, [])

    def delMember(self, username):
        self.portal.portal_membership.deleteMembers([username])

    def test_initHistory(self):
        request = self.app.REQUEST
        context = getattr(self.folder, "d1")
        viewlet = WorkflowHistoryViewlet(context, request, None, None)
        viewlet.update()
        history = viewlet.workflowHistory()
        self.assertEqual(len(history), 1)
        self.assertEqual(history[0]["action"], None)

    def test_transitionHistory(self):
        wf_tool = self.portal.portal_workflow
        request = self.app.REQUEST
        context = getattr(self.folder, "d1")
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        wf_tool.doActionFor(context, "publish")

        viewlet = WorkflowHistoryViewlet(context, request, None, None)
        viewlet.update()

        history = viewlet.workflowHistory()
        self.assertEqual(len(history), 2)
        self.assertEqual(history[0]["action"], "publish")

        # add a temporary user to perform a transition
        self.addMember("tempuser", roles=("Member", "Manager"))
        login(self.portal, "tempuser")
        wf_tool.doActionFor(context, action="retract", actor=None)
        logout()

        login(self.portal, TEST_USER_NAME)

        # remove the user
        self.delMember("tempuser")

        # if the user that performed the transition no longer exists, the link
        # shouldn't be included.
        viewlet = WorkflowHistoryViewlet(context, request, None, None)
        viewlet.update()
        history = viewlet.workflowHistory()

        self.assertEqual(len(history), 3)
        self.assertEqual(history[0]["actor_home"], "")


class TestContentHistoryViewlet(ViewletsFunctionalTestCase):
    def setUp(self):
        super().setUp()
        # add document, perform transition, set history for non-existent
        # member and also None (anonymous)
        self.folder.invokeFactory("Document", "d1")

    def test_initialHistory(self):
        request = self.app.REQUEST
        context = getattr(self.folder, "d1")
        viewlet = ContentHistoryViewlet(context, request, None, None)
        viewlet.update()
        self.assertEqual(len(viewlet.revisionHistory()), 1)

    def test_revisionHistory(self):
        repo_tool = self.portal.portal_repository
        request = self.app.REQUEST
        context = getattr(self.folder, "d1")
        viewlet = ContentHistoryViewlet(context, request, None, None)
        viewlet.update()

        # after creating the item the is a first revision
        history = viewlet.revisionHistory()
        self.assertEqual(len(history), 1)

        repo_tool.save(context, comment="Initial Revision")
        viewlet.update()
        history = viewlet.revisionHistory()
        self.assertEqual(len(history), 2)
        self.assertEqual(history[0]["comments"], "Initial Revision")

        repo_tool.save(context, comment="Second Revision")
        viewlet.update()
        history = viewlet.revisionHistory()
        self.assertTrue(
            "http://nohost/plone/Members/test_user_1_/d1/@@history?one=2&two=1"
            in history[0]["diff_previous_url"]
        )

        # check diff link does not appear if content is not diffable
        diff_tool = self.portal.portal_diff
        diff_tool.setDiffForPortalType("Document", {})
        viewlet.update()
        history = viewlet.revisionHistory()
        self.assertFalse("diff_previous_url" in history[0])

    def test_revertAbility(self):
        # check revert URL is generated only if the user has the appropriate
        # permission
        repo_tool = self.portal.portal_repository
        request = self.app.REQUEST
        context = getattr(self.folder, "d1")
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        repo_tool.save(context, comment="Initial Revision")
        repo_tool.save(context, comment="Second Revision")

        viewlet = ContentHistoryViewlet(context, request, None, None)

        viewlet.update()
        history = viewlet.revisionHistory()
        self.assertTrue(
            "http://nohost/plone/Members/test_user_1_/d1/revertversion"
            in history[0]["revert_url"]
        )  # noqa

        self.portal.manage_permission(
            "CMFEditions: Revert to previous versions", [], False
        )

        viewlet.update()
        history = viewlet.revisionHistory()
        self.assertEqual(history[0]["revert_url"], None)



from plone.app.layout.testing import FUNCTIONAL_TESTING
from plone.app.layout.testing import INTEGRATION_TESTING
from plone.app.layout.testing import TEST_USER_ID

import unittest


class ViewletsTestCase(unittest.TestCase):
    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.app = self.layer["app"]
        self.folder = self.portal["Members"][TEST_USER_ID]


class ViewletsFunctionalTestCase(unittest.TestCase):
    layer = FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.app = self.layer["app"]
        self.folder = self.portal["Members"][TEST_USER_ID]



from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getMultiAdapter
from zope.interface import implementer
from zope.viewlet.interfaces import IViewlet


@implementer(IViewlet)
class LockInfoViewlet(BrowserView):
    template = ViewPageTemplateFile("locking.pt")

    def __init__(self, context, request, view, manager):
        super().__init__(context, request)
        self.__parent__ = view
        self.context = context
        self.request = request
        self.view = view
        self.manager = manager
        self.info = getMultiAdapter((context, request), name="plone_lock_info")

    def update(self):
        pass

    def render(self):
        return self.template()

    def lock_is_stealable(self):
        return self.info.lock_is_stealable()

    def lock_info(self):
        return self.info.lock_info()



from AccessControl import getSecurityManager
from Acquisition import aq_base
from Acquisition import aq_inner
from collections import defaultdict
from functools import total_ordering
from html import escape
from plone.app.layout.globals.interfaces import IViewView
from plone.base import PloneMessageFactory as _
from plone.base.interfaces import IPloneSiteRoot
from plone.base.interfaces import ISearchSchema
from plone.base.interfaces import ISiteSchema
from plone.base.interfaces.controlpanel import INavigationSchema
from plone.base.navigationroot import get_navigation_root
from plone.base.utils import safe_text
from plone.i18n.interfaces import ILanguageSchema
from plone.memoize.view import memoize
from plone.protect.utils import addTokenToUrl
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import getSiteLogo
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from urllib.parse import unquote
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.i18n import translate
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.viewlet.interfaces import IViewlet

import json
import zope.deferredimport


zope.deferredimport.initialize()
zope.deferredimport.deprecated(
    "Import from plone.app.portlets.browser.viewlets instead",
    ManagePortletsFallbackViewlet="plone.app.portlets.browser.viewlets:ManagePortletsFallbackViewlet",
    FooterViewlet="plone.app.portlets.browser.viewlets:FooterViewlet",
)


@implementer(IViewlet)
@total_ordering
class ViewletBase(BrowserView):
    """Base class with common functions for link viewlets."""

    def __init__(self, context, request, view, manager=None):
        super().__init__(context, request)
        self.__parent__ = view
        self.context = context
        self.request = request
        self.view = view
        self.manager = manager

    def __hash__(self):
        return id(self) * 16

    def update(self):
        self.portal_state = getMultiAdapter(
            (self.context, self.request), name="plone_portal_state"
        )
        self.site_url = self.portal_state.portal_url()
        self.navigation_root_url = self.portal_state.navigation_root_url()

    def render(self):
        # defer to index method, because that's what gets overridden by the
        # template ZCML attribute
        return self.index()

    def index(self):
        raise NotImplementedError("`index` method must be implemented by subclass.")

    def __lt__(self, other):
        """Sort by name"""
        return self.__name__ < other.__name__

    def __eq__(self, other):
        """Check for equality"""
        return id(self) == id(other)


class TitleViewlet(ViewletBase):
    index = ViewPageTemplateFile("title.pt")

    # separator of page- and portal-title
    sep = " &mdash; "

    @property
    @memoize
    def site_title_setting(self):
        registry = getUtility(IRegistry)
        site_settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        return site_settings.site_title

    @property
    @memoize
    def page_title(self):
        """
        Get the page title. If we are in the portal_factory we want use the
        "Add $FTI_TITLE" form (see #12117).

        NOTE: other implementative options can be:
         - to use "Untitled" instead of "Add" or
         - to check the isTemporary method of the edit view instead of the
           creation_flag
        """
        if hasattr(aq_base(self.context), "isTemporary") and self.context.isTemporary():
            # if we are in the portal_factory we want the page title to be
            # "Add fti title"
            portal_types = getToolByName(self.context, "portal_types")
            fti = portal_types.getTypeInfo(self.context)
            return translate(
                "heading_add_item",
                domain="plone",
                mapping={"itemtype": fti.Title()},
                context=self.request,
                default="Add ${itemtype}",
            )

        # If we are on portal root, look up the portal title from registry
        if IPloneSiteRoot.providedBy(self.context):
            return self.site_title_setting

        context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        return escape(safe_text(context_state.object_title()))

    def update(self):
        if IPloneSiteRoot.providedBy(self.context):
            self.site_title = self.site_title_setting
            return
        portal_state = getMultiAdapter(
            (self.context, self.request), name="plone_portal_state"
        )
        if IPloneSiteRoot.providedBy(portal_state.navigation_root()):
            portal_title = self.site_title_setting
        else:
            portal_title = escape(safe_text(portal_state.navigation_root_title()))
        if self.page_title == portal_title:
            self.site_title = portal_title
        else:
            self.site_title = self.sep.join([self.page_title, portal_title])


class DublinCoreViewlet(ViewletBase):
    index = ViewPageTemplateFile("dublin_core.pt")

    def update(self):
        plone_utils = getToolByName(self.context, "plone_utils")
        context = aq_inner(self.context)
        self.metatags = plone_utils.listMetaTags(context).items()


class TableOfContentsViewlet(ViewletBase):
    index = ViewPageTemplateFile("toc.pt")

    def update(self):
        self.enabled = getattr(aq_base(self.context), "table_of_contents", False)


class SiteActionsViewlet(ViewletBase):
    index = ViewPageTemplateFile("site_actions.pt")

    def update(self):
        context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        self.site_actions = context_state.actions("site_actions")


class SearchBoxViewlet(ViewletBase):
    index = ViewPageTemplateFile("searchbox.pt")

    def update(self):
        super().update()

        context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )

        registry = getUtility(IRegistry)
        search_settings = registry.forInterface(ISearchSchema, prefix="plone")
        self.livesearch = search_settings.enable_livesearch
        self.show_images = search_settings.search_show_images

        folder = context_state.folder()
        self.folder_path = "/".join(folder.getPhysicalPath())


class LogoViewlet(ViewletBase):
    index = ViewPageTemplateFile("logo.pt")

    def update(self):
        super().update()

        # TODO: should this be changed to settings.site_title?
        self.navigation_root_title = self.portal_state.navigation_root_title()

        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        self.logo_title = settings.site_title
        self.img_src = getSiteLogo()


class GlobalSectionsViewlet(ViewletBase):
    index = ViewPageTemplateFile("sections.pt")

    _opener_markup_template = (
        '<input id="navitem-{uid}" type="checkbox" class="opener" />'
        '<label for="navitem-{uid}" role="button" aria-label="{title}"></label>'  # noqa: E 501
    )
    _item_markup_template = (
        '<li class="{id}{has_sub_class} nav-item">'
        '<a href="{url}" class="state-{review_state} nav-link"{aria_haspopup}>{title}</a>{opener}'  # noqa: E 501
        "{sub}"
        "</li>"
    )
    _subtree_markup_wrapper = '<ul class="has_subtree dropdown">{out}</ul>'

    def __init__(self, *args):
        super().__init__(*args)
        self.registry = getUtility(IRegistry)

    @property
    def settings(self):
        return self.registry.forInterface(INavigationSchema, prefix="plone")

    @property
    def language_settings(self):
        return self.registry.forInterface(ILanguageSchema, prefix="plone")

    @property
    def navtree_path(self):
        return get_navigation_root(self.context)

    @property
    def current_language(self):
        language_settings = self.registry.forInterface(ILanguageSchema, prefix="plone")
        return (
            self.request.get("LANGUAGE", None)
            or (self.context and aq_inner(self.context).Language())
            or language_settings.default_language
        )

    @property
    def types_using_view(self):
        return self.registry.get("plone.types_use_view_action_in_listings", [])

    @property
    @memoize
    def navtree(self):
        ret = defaultdict(list)
        navtree_path = self.navtree_path
        portal_tabs = self.portal_tabs
        for tab in portal_tabs:
            entry = tab.copy()
            entry.update(
                {
                    "path": "/".join((navtree_path, tab["id"])),
                    "uid": tab["id"],
                }
            )
            if "review_state" not in entry:
                entry["review_state"] = None

            if "title" not in entry:
                entry["title"] = tab.get("name") or tab.get("description") or tab["id"]
            else:
                # translate Home tab
                entry["title"] = translate(
                    entry["title"], domain="plone", context=self.request
                )

            entry["title"] = escape(safe_text(entry["title"]))
            if "name" in entry and entry["name"]:
                entry["name"] = escape(safe_text(entry["name"]))
            self.customize_tab(entry, tab)
            ret[navtree_path].append(entry)

        settings = self.settings
        if not settings.generate_tabs:
            return ret

        query = {
            "path": {
                "query": self.navtree_path,
                "depth": settings.navigation_depth,
            },
            "portal_type": {"query": settings.displayed_types},
            "Language": self.current_language,
            "sort_on": settings.sort_tabs_on,
            "is_default_page": False,
        }

        if settings.sort_tabs_reversed:
            query["sort_order"] = "reverse"

        if not settings.nonfolderish_tabs:
            query["is_folderish"] = True

        if settings.filter_on_workflow:
            query["review_state"] = list(settings.workflow_states_to_show or ())

        if not settings.show_excluded_items:
            query["exclude_from_nav"] = False

        context_path = "/".join(self.context.getPhysicalPath())
        self.customize_query(query)
        portal_catalog = getToolByName(self.context, "portal_catalog")
        brains = portal_catalog.searchResults(**query)

        types_using_view = set(self.types_using_view)
        for brain in brains:
            brain_path = brain.getPath()
            brain_parent_path = brain_path.rpartition("/")[0]
            if portal_tabs and brain_parent_path == navtree_path:
                # This should be already provided by the portal_tabs_view
                continue
            if brain.exclude_from_nav and not context_path.startswith(brain_path):
                # skip excluded items if they're not in our context path
                continue
            url = brain.getURL()
            if brain.portal_type in types_using_view:
                url += "/view"
            entry = {
                "id": brain.getId,
                "path": brain_path,
                "uid": brain.UID,
                "url": url,
                "title": escape(safe_text(brain.Title)),
                "review_state": brain.review_state,
            }
            self.customize_entry(entry, brain)
            ret[brain_parent_path].append(entry)

        return ret

    def customize_query(self, query):
        """Helper to customize the catalog query."""
        pass

    def customize_tab(self, entry, tab):
        """Helper to add custom entry keys/values."""
        pass

    def customize_entry(self, entry, brain):
        """Helper to add custom entry keys/values."""
        pass

    def render_item(self, item, path):
        sub = self.build_tree(item["path"], first_run=False)
        if sub:
            item.update(
                {
                    "sub": sub,
                    "opener": self._opener_markup_template.format(**item),
                    "aria_haspopup": ' aria-haspopup="true"',
                    "has_sub_class": " has_subtree",
                }
            )
        else:
            item.update(
                {
                    "sub": sub,
                    "opener": "",
                    "aria_haspopup": "",
                    "has_sub_class": "",
                }
            )
        return self._item_markup_template.format(**item)

    def build_tree(self, path, first_run=True):
        """Non-template based recursive tree building.
        3-4 times faster than template based.
        """
        out = ""
        for item in self.navtree.get(path, []):
            out += self.render_item(item, path)

        if not first_run and out:
            out = self._subtree_markup_wrapper.format(out=out)
        return out

    def render_globalnav(self):
        return self.build_tree(self.navtree_path)

    @property
    @memoize
    def portal_tabs(self):
        portal_tabs_view = getMultiAdapter(
            (self.context, self.request), name="portal_tabs_view"
        )
        return portal_tabs_view.topLevelTabs()


class PersonalBarViewlet(ViewletBase):
    homelink_url = ""
    user_name = ""

    def update(self):
        super().update()
        context = aq_inner(self.context)

        context_state = getMultiAdapter(
            (context, self.request), name="plone_context_state"
        )

        user_actions = context_state.actions("user")
        self.user_actions = []
        for action in user_actions:
            info = {
                "title": action["title"],
                "href": action["url"],
                "id": "personaltools-{}".format(action["id"]),
                "target": action.get("link_target", None),
                "icon": action.get("icon", None),
            }
            modal = action.get("modal")
            if modal:
                info["class"] = "pat-plone-modal"
                if '"title":' in modal:
                    modal = json.loads(modal)
                    modal["title"] = translate(_(modal["title"]), context=self.request)
                    modal = json.dumps(modal)

                info["data-pat-plone-modal"] = modal
            self.user_actions.append(info)

        self.anonymous = self.portal_state.anonymous()

        if not self.anonymous:
            member = self.portal_state.member()
            userid = member.getId()

            self.homelink_url = f"{self.navigation_root_url}/useractions"

            membership = getToolByName(context, "portal_membership")
            member_info = membership.getMemberInfo(userid)
            # member_info is None if there's no Plone user object, as when
            # using OpenID.
            if member_info:
                fullname = member_info.get("fullname", "")
            else:
                fullname = None
            if fullname:
                self.user_name = fullname
            else:
                self.user_name = userid


class ContentViewsViewlet(ViewletBase):
    index = ViewPageTemplateFile("contentviews.pt")
    menu_template = ViewPageTemplateFile("menu.pt")

    default_tab = "nothing"
    primary = ["folderContents", "edit", "view"]

    def update(self):
        # The drop-down menus are pulled in via a simple content provider
        # from plone.app.contentmenu. This behaves differently depending on
        # whether the view is marked with IViewView. If our parent view
        # provides that marker, we should do it here as well.
        super().update()
        if IViewView.providedBy(self.__parent__):
            alsoProvides(self, IViewView)

        self.tabSet1, self.tabSet2 = self.getTabSets()

    @memoize
    def getTabSets(self):
        context = aq_inner(self.context)
        context_url = context.absolute_url()
        context_fti = context.getTypeInfo()

        context_state = getMultiAdapter(
            (context, self.request), name="plone_context_state"
        )
        actions = context_state.actions

        action_list = []
        if context_state.is_structural_folder():
            action_list = actions("folder")
        action_list.extend(actions("object"))
        action_list.extend(actions("object_actions"))

        tabSet1 = []
        tabSet2 = []
        found_selected = False
        fallback_action = None

        try:
            request_url = self.request["ACTUAL_URL"]
        except KeyError:
            # not a real request, could be a test. Let's not fail.
            request_url = context_url
        request_url_path = request_url[len(context_url) :]

        if request_url_path.startswith("/"):
            request_url_path = request_url_path[1:]

        for item in action_list:
            item.update({"selected": False})

            action_url = item["url"].strip()
            starts = action_url.startswith
            if starts("http") or starts("javascript"):
                item["url"] = action_url
            else:
                item["url"] = f"{context_url}/{action_url}"
            item["url"] = addTokenToUrl(item["url"], self.request)

            action_method = item["url"].split("/")[-1].split("?")[0]

            # Action method may be a method alias:
            # Attempt to resolve to a template.
            action_method = context_fti.queryMethodID(
                action_method, default=action_method
            )
            if action_method:
                request_action = unquote(request_url_path).split("?")[0]
                request_action = context_fti.queryMethodID(
                    request_action, default=request_action
                )
                if action_method == request_action and item["id"] != "view":
                    item["selected"] = True
                    found_selected = True

            current_id = item["id"]
            if current_id == self.default_tab:
                fallback_action = item

            modal = item.get("modal", None)
            item["cssClass"] = ""
            if modal:
                item["cssClass"] += " pat-plone-modal"
                if "ajax_load" not in item["url"]:
                    item["url"] += "?ajax_load=1"

            if item["id"] in self.primary:
                tabSet1.append(item)
            else:
                tabSet2.append(item)

        if not found_selected and fallback_action is not None:
            fallback_action["selected"] = True

        tabSet1.sort(key=lambda item: self.primary.index(item["id"]))
        return tabSet1, tabSet2

    def locked_icon(self):
        if not getSecurityManager().checkPermission(
            "Modify portal content", self.context
        ):
            return ""

        locked = False
        lock_info = queryMultiAdapter(
            (self.context, self.request), name="plone_lock_info"
        )
        if lock_info is not None:
            locked = lock_info.is_locked()
        else:
            context = aq_inner(self.context)
            lockable = getattr(context.aq_explicit, "wl_isLocked", None) is not None
            locked = lockable and context.wl_isLocked()

        if not locked:
            return ""

        portal = self.portal_state.portal()
        icon = portal.restrictedTraverse("lock_icon.png")
        return icon.tag(title="Locked")


class PathBarViewlet(ViewletBase):
    index = ViewPageTemplateFile("path_bar.pt")

    def update(self):
        super().update()

        self.is_rtl = self.portal_state.is_rtl()

        breadcrumbs_view = getMultiAdapter(
            (self.context, self.request), name="breadcrumbs_view"
        )
        self.breadcrumbs = breadcrumbs_view.breadcrumbs()


class TinyLogoViewlet(ViewletBase):
    index = ViewPageTemplateFile("tiny_logo.pt")



from plone.app.layout.testing import INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing.helpers import logout
from plone.base.interfaces import INavigationRoot
from plone.base.interfaces.controlpanel import ILinkSchema
from plone.portlets.interfaces import IPortletType
from plone.registry.interfaces import IRegistry
from zope.component import getMultiAdapter
from zope.component import getUtility

import os
import unittest
import zope.interface


class TestLayoutView(unittest.TestCase):
    """Tests the global layout view."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.view = self.portal.restrictedTraverse("@@plone_layout")
        self.portal.invokeFactory("Folder", "folder1")
        self.portal.invokeFactory("Document", "front-page")
        self.portal.setDefaultPage("front-page")

    def testHavePortlets(self):
        have_portlets = self.view.have_portlets
        self.assertEqual(False, have_portlets("plone.leftcolumn"))
        # We have no portlet on the right in Plone 5.0
        self.assertEqual(False, have_portlets("plone.rightcolumn"))

    def testEnableColumns(self):
        # Make sure we can force a column to appear even if there are no
        # portlets
        self.app = self.layer["app"]
        self.app.REQUEST.set("disable_plone.leftcolumn", 0)
        self.assertEqual(True, self.view.have_portlets("plone.leftcolumn"))

    def testDisableColumns(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.app = self.layer["app"]
        # Now add some portlets to be sure we have columns.  For
        # simplicity we want a portlet that has no add form.  Note
        # that apparently the Calendar had no add form until Plone
        # 4.3, but since 4.4 it does, so it is not fit to use here.
        portlet = getUtility(IPortletType, name="portlets.Login")
        mapping_left = self.portal.restrictedTraverse(
            "++contextportlets++plone.leftcolumn"
        )
        mapping_right = self.portal.restrictedTraverse(
            "++contextportlets++plone.rightcolumn"
        )
        for m in mapping_left.keys():
            del mapping_left[m]
        addview_left = mapping_left.restrictedTraverse("+/" + portlet.addview)

        for m in mapping_right.keys():
            del mapping_right[m]
        addview_right = mapping_right.restrictedTraverse("+/" + portlet.addview)

        # This is a NullAddForm - calling it does the work
        addview_left()
        addview_right()

        # Logout, otherwise the login portlet will never show.
        logout()

        # Check that we do not explicitly disable the columns.  This
        # may happen if we change the request in this method or if the
        # addviews return an add-form after all.
        self.assertTrue("disable_plone.leftcolumn" not in self.app.REQUEST)
        self.assertTrue("disable_plone.rightcolumn" not in self.app.REQUEST)

        self.assertEqual(True, self.view.have_portlets("plone.leftcolumn"))
        self.app.REQUEST.set("disable_plone.leftcolumn", 1)
        self.assertEqual(False, self.view.have_portlets("plone.leftcolumn"))

        self.assertEqual(True, self.view.have_portlets("plone.rightcolumn"))
        self.app.REQUEST.set("disable_plone.rightcolumn", 1)
        self.assertEqual(False, self.view.have_portlets("plone.rightcolumn"))

    def testBodyClass(self):
        context = self.portal["front-page"]
        view = context.restrictedTraverse("view")
        layout_view = context.restrictedTraverse("@@plone_layout")
        body_class = layout_view.bodyClass(view, layout_view)
        assert "section-front-page" in body_class

    def testBodyClassTemplate(self):
        # test of first parameter only
        context = self.portal["front-page"]

        view = context.restrictedTraverse("view")
        layout_view = context.restrictedTraverse("@@plone_layout")

        # case 1: name from first parameter, expected a template or view
        from Products.CMFCore.FSPageTemplate import FSPageTemplate

        template = FSPageTemplate(
            "document_view",
            os.path.join(os.path.dirname(__file__), "data", "bodyclass_nametest.pt"),
        )
        body_class = layout_view.bodyClass(template, view)
        self.assertIn("template-document_view", body_class)

        # case 2: even w/o second parameter it has to work
        body_class = layout_view.bodyClass(template, None)
        self.assertIn("template-document_view", body_class)

        # case 3: if there's no template get name from view
        body_class = layout_view.bodyClass(None, view)
        self.assertIn("template-view", body_class)

    def testBodyClassWithNavigationRoot(self):
        # mark a folder "between" self.folder and self.portal with
        # INavigationRoot
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        zope.interface.alsoProvides(self.portal.folder1, INavigationRoot)
        context = self.portal.folder1
        view = context.restrictedTraverse("@@plone_layout")
        template = context.restrictedTraverse("view")
        body_class = view.bodyClass(template, view)
        assert "site-%s" % context.getId() in body_class

    def testBodyClassWithEverySection(self):
        # mark a folder "between" self.folder and self.portal with
        # INavigationRoot
        zope.interface.alsoProvides(self.portal.folder1, INavigationRoot)
        self.portal.folder1.invokeFactory("Folder", "folder2")
        self.portal.folder1.folder2.invokeFactory("Folder", "folder3")
        self.portal.folder1.folder2.folder3.invokeFactory("Document", "page")
        context = self.portal.folder1.folder2.folder3.page
        view = context.restrictedTraverse("@@plone_layout")
        template = context.restrictedTraverse("view")
        body_class = view.bodyClass(template, view)
        assert "section-folder2 site-folder1" in body_class
        assert " subsection-folder3 subsection-folder3-page" in body_class

    def testBodyClassWithEverySectionTurnedOff(self):
        registry = getUtility(IRegistry)
        registry["plone.app.layout.globals.bodyClass.depth"] = 0
        zope.interface.alsoProvides(self.portal.folder1, INavigationRoot)
        self.portal.folder1.invokeFactory("Folder", "folder2")
        self.portal.folder1.folder2.invokeFactory("Folder", "folder3")
        self.portal.folder1.folder2.folder3.invokeFactory("Document", "page")
        context = self.portal.folder1.folder2.folder3.page
        view = context.restrictedTraverse("@@plone_layout")
        template = context.restrictedTraverse("view")
        body_class = view.bodyClass(template, view)
        assert "subsection-folder2 subsection-folder2-folder3" not in body_class
        assert " subsection-folder2-folder3-page" not in body_class

    def testBodyClassWithMarkSpecialLinksOnOff(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ILinkSchema, prefix="plone", check=False)
        context = self.portal["front-page"]
        template = context.restrictedTraverse("view")
        view = context.restrictedTraverse("@@plone_layout")

        # Case 1
        settings.mark_special_links = False
        settings.external_links_open_new_window = False
        body_class = view.bodyClass(template, view)
        self.assertTrue("pat-markspeciallinks" not in body_class)

        # Case 2
        settings.mark_special_links = True
        settings.external_links_open_new_window = False
        body_class = view.bodyClass(template, view)
        self.assertTrue("pat-markspeciallinks" in body_class)

        # Case 3
        settings.mark_special_links = False
        settings.external_links_open_new_window = True
        body_class = view.bodyClass(template, view)
        self.assertTrue("pat-markspeciallinks" in body_class)

        # Case 4
        settings.mark_special_links = True
        settings.external_links_open_new_window = True
        body_class = view.bodyClass(template, view)
        self.assertTrue("pat-markspeciallinks" in body_class)

    def testBodyClassColMarker(self):
        context = self.portal["front-page"]
        view = context.restrictedTraverse("view")
        layout_view = context.restrictedTraverse("@@plone_layout")
        body_class = layout_view.bodyClass(view, layout_view)
        assert "col-content" in body_class


class TestLayoutViewXHR(unittest.TestCase):
    """Tests the global layout view with it's XHR utilities."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"].clone()

    @property
    def view(self):
        return getMultiAdapter((self.portal, self.request), name="plone_layout")

    def test_is_ajax_default(self):
        """Test the default case where no XHR request is requested."""
        self.assertFalse(self.view.is_ajax)

    def test_is_ajax_ajax(self):
        """Test, if the view correctly identifies XHR requests."""
        self.request.environ["HTTP_X_REQUESTED_WITH"] = "XMLHttpRequest"
        self.assertTrue(self.view.is_ajax)



from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.dexterity.behaviors.nextprevious import INextPreviousProvider
from plone.app.layout.viewlets import ViewletBase
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
from zope.component import getMultiAdapter


class NextPreviousView(BrowserView):
    """Information about next/previous navigation"""

    def next(self):
        provider = self._provider()
        if provider is None:
            return None
        return provider.getNextItem(aq_inner(self.context))

    def previous(self):
        provider = self._provider()
        if provider is None:
            return None
        return provider.getPreviousItem(aq_inner(self.context))

    def enabled(self):
        provider = self._provider()
        if provider is None:
            return False
        return provider.enabled

    def _provider(self):
        # Note - the next/previous provider is the container of this object!
        # This may not support next/previous navigation, so code defensively
        return INextPreviousProvider(aq_parent(aq_inner(self.context)), None)

    def isViewTemplate(self):
        plone = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        return plone.is_view_template()


class NextPreviousViewlet(ViewletBase, NextPreviousView):
    index = ZopeTwoPageTemplateFile("nextprevious.pt")


class NextPreviousLinksViewlet(ViewletBase, NextPreviousView):
    index = ZopeTwoPageTemplateFile("links.pt")



from plone.app.layout.testing import INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing.helpers import logout
from plone.base.interfaces import INavigationRoot
from plone.base.interfaces import INonStructuralFolder
from plone.base.utils import unrestricted_construct_instance
from plone.locking.interfaces import ILockable
from Products.CMFDynamicViewFTI.interfaces import IBrowserDefault
from zope.interface import alsoProvides
from zope.interface import directlyProvides

import unittest


class TestContextStateView(unittest.TestCase):
    """Ensure that the basic redirector setup is successful."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory("Folder", "f0")
        self.folder = self.portal["f0"]
        self.fview = self.folder.restrictedTraverse("@@plone_context_state")
        self.folder.invokeFactory("Document", "d1")
        self.folder.setDefaultPage("d1")
        self.dview = self.folder.d1.restrictedTraverse("@@plone_context_state")
        self.folder.invokeFactory("Folder", "f1")
        directlyProvides(self.folder.f1, INonStructuralFolder)
        self.sview = self.folder.f1.restrictedTraverse("@@plone_context_state")

        self.pview = self.portal.restrictedTraverse("@@plone_context_state")

    def test_current_page_url(self):
        url = self.folder.absolute_url() + "/some_view"
        self.app.REQUEST["ACTUAL_URL"] = url
        self.app.REQUEST["QUERY_STRING"] = "foo=bar"
        self.assertEqual(self.fview.current_page_url(), url + "?foo=bar")

    def test_current_base_url(self):
        url = self.folder.absolute_url() + "/some_view"
        self.app.REQUEST["ACTUAL_URL"] = url
        self.app.REQUEST["QUERY_STRING"] = "foo=bar"
        self.assertEqual(self.fview.current_base_url(), url)

    def test_canonical_object(self):
        self.assertEqual(self.fview.canonical_object(), self.folder)
        self.assertEqual(self.dview.canonical_object(), self.folder)

    def test_canonical_object_url(self):
        self.assertEqual(self.fview.canonical_object_url(), self.folder.absolute_url())
        self.assertEqual(self.dview.canonical_object_url(), self.folder.absolute_url())

    def test_view_url(self):
        self.assertEqual(self.fview.view_url(), self.folder.absolute_url())
        self.assertEqual(self.dview.view_url(), self.folder.d1.absolute_url())
        self.folder.invokeFactory("File", "file1")
        self.fileview = self.folder.file1.restrictedTraverse("@@plone_context_state")
        self.assertEqual(
            self.fileview.view_url(), self.folder.file1.absolute_url() + "/view"
        )

    def test_view_template_id(self):
        self.folder.setLayout("foo_view")
        self.assertEqual(self.fview.view_template_id(), "foo_view")

    def test_view_template_id_nonbrowserdefault(self):
        # The view template id is taken from the FTI for non-browserdefault
        # (non ATContentTypes) content
        tf = unrestricted_construct_instance("TempFolder", self.folder, "tf")
        tfview = tf.restrictedTraverse("@@plone_context_state")
        self.assertEqual(tfview.view_template_id(), "index_html")

    def test_view_template_id_nonbrowserdefault_nonempty(self):
        # The view template id is taken from the FTI for non-browserdefault
        # (non ATContentTypes) content. In this case the default view action
        # includes an actual template name

        # Set the expression to include a view name.
        fti = self.portal.portal_types.TempFolder
        view_action = fti.getActionObject("object/view")
        view_expression = view_action.getActionExpression()
        view_action.setActionExpression("foobar")

        tf = unrestricted_construct_instance("TempFolder", self.folder, "tf")
        tf.manage_addLocalRoles(TEST_USER_ID, ("Manager",))
        tfview = tf.restrictedTraverse("@@plone_context_state")
        self.assertEqual(tfview.view_template_id(), "foobar")

        # Reset the FTI action expression
        view_action.setActionExpression(view_expression)

    def test_view_template_id_nonbrowserdefault_restricted(self):
        # The view template id is taken from the FTI for non-browserdefault
        # (non ATContentTypes) content. If the view action is protected by
        # a non-default permission, this should still work if the current
        # user has the right permission, locally.

        # Set access to something the default user does not have, normally
        fti = self.portal.portal_types.TempFolder
        view_action = fti.getActionObject("object/view")
        view_perms = view_action.getPermissions()
        view_action.edit(permissions=("Modify Portal Content",))

        tf = unrestricted_construct_instance("TempFolder", self.folder, "tf")
        tf.manage_addLocalRoles(TEST_USER_ID, ("Manager",))
        tfview = tf.restrictedTraverse("@@plone_context_state")
        self.assertEqual(tfview.view_template_id(), "index_html")

        # Reset the FTI permissions
        view_action.edit(permissions=view_perms)

    def test_is_view_template_default_page(self):
        self.app.REQUEST["ACTUAL_URL"] = self.folder.absolute_url()
        # Whether you're viewing the folder or its default page ...
        self.assertEqual(self.fview.is_view_template(), True)
        self.assertEqual(self.dview.is_view_template(), True)

    def test_is_view_template_trailing_slash(self):
        self.app.REQUEST["ACTUAL_URL"] = self.folder.absolute_url() + "/"
        self.assertEqual(self.fview.is_view_template(), True)
        self.assertEqual(self.dview.is_view_template(), True)

    def test_is_view_template_template(self):
        self.folder.setLayout("foo_view")
        self.app.REQUEST["ACTUAL_URL"] = self.folder.absolute_url() + "/foo_view"
        self.assertEqual(self.fview.is_view_template(), True)
        self.assertEqual(self.dview.is_view_template(), False)

    def test_is_view_template_template_z3view(self):
        self.folder.setLayout("foo_view")
        self.app.REQUEST["ACTUAL_URL"] = self.folder.absolute_url() + "/@@foo_view"
        self.assertEqual(self.fview.is_view_template(), True)
        self.assertEqual(self.dview.is_view_template(), False)

    def test_is_view_template_view(self):
        self.folder.setLayout("foo_view")
        self.app.REQUEST["ACTUAL_URL"] = self.folder.absolute_url() + "/view"
        self.assertEqual(self.fview.is_view_template(), True)
        self.assertEqual(self.dview.is_view_template(), False)

    def test_is_view_template_other(self):
        self.folder.setLayout("foo_view")
        self.app.REQUEST["ACTUAL_URL"] = self.folder.absolute_url() + "/bar_view"
        self.assertEqual(self.fview.is_view_template(), False)
        self.assertEqual(self.dview.is_view_template(), False)

    def test_is_view_template_edit(self):
        self.app.REQUEST["ACTUAL_URL"] = self.folder.absolute_url() + "/edit"
        self.assertEqual(self.fview.is_view_template(), False)
        self.assertEqual(self.dview.is_view_template(), False)

    def test_is_view_template_alias(self):
        browserDefault = IBrowserDefault(self.folder, None)
        fti = browserDefault.getTypeInfo()
        aliases = fti.getMethodAliases()
        aliases["foo_alias"] = "(Default)"
        fti.setMethodAliases(aliases)
        self.app.REQUEST["ACTUAL_URL"] = self.folder.absolute_url() + "/foo_alias"
        self.assertEqual(self.fview.is_view_template(), True)
        self.assertEqual(self.dview.is_view_template(), False)

    def test_object_url(self):
        self.assertEqual(self.fview.object_url(), self.folder.absolute_url())
        self.assertEqual(self.dview.object_url(), self.folder.d1.absolute_url())

    def test_object_title(self):
        self.folder.d1.setTitle("My title")
        self.assertEqual(self.dview.object_title(), "My title")

    def test_workflow_state(self):
        wfstate = self.portal.portal_workflow.getInfoFor(self.folder.d1, "review_state")
        self.assertEqual(self.dview.workflow_state(), wfstate)

    def test_parent(self):
        self.assertEqual(self.dview.parent(), self.folder)
        self.assertEqual(self.sview.parent(), self.folder)

    def test_folder(self):
        self.assertEqual(self.fview.folder(), self.folder)
        self.assertEqual(self.dview.folder(), self.folder)
        self.assertEqual(self.sview.folder(), self.folder)

    def test_is_folderish(self):
        self.assertEqual(self.fview.is_folderish(), True)
        self.assertEqual(self.dview.is_folderish(), False)
        self.assertEqual(self.sview.is_folderish(), True)

    def test_is_structural_folder(self):
        self.assertEqual(self.fview.is_structural_folder(), True)
        self.assertEqual(self.dview.is_structural_folder(), False)
        self.assertEqual(self.sview.is_structural_folder(), False)

    def test_is_default_page(self):
        self.assertEqual(self.fview.is_default_page(), False)
        self.assertEqual(self.dview.is_default_page(), True)
        self.assertEqual(self.sview.is_default_page(), False)

    def test_is_portal_root(self):
        self.assertEqual(self.fview.is_portal_root(), False)
        self.assertEqual(self.dview.is_portal_root(), False)
        self.assertEqual(self.sview.is_portal_root(), False)
        self.assertEqual(self.pview.is_portal_root(), True)

    def test_is_navigation_root(self):
        alsoProvides(self.folder, INavigationRoot)
        self.assertEqual(self.fview.is_navigation_root(), True)
        self.assertEqual(self.dview.is_navigation_root(), True)
        self.assertEqual(self.sview.is_navigation_root(), False)
        self.assertEqual(self.pview.is_navigation_root(), True)

    def test_is_editable(self):
        self.assertEqual(self.dview.is_editable(), True)
        logout()
        del self.app.REQUEST.__annotations__
        self.assertEqual(self.dview.is_editable(), False)

    def test_is_locked(self):
        self.assertEqual(self.dview.is_locked(), False)
        ILockable(self.folder.d1).lock()
        logout()
        # The object is not "locked" if it was locked by the
        # current user
        del self.app.REQUEST.__annotations__
        self.assertEqual(self.dview.is_locked(), True)

    def test_actions(self):
        actions = self.fview.actions("user")
        self.assertTrue(actions[0]["category"] == "user")



from AccessControl import Unauthorized
from Acquisition import aq_inner
from plone.app.layout.viewlets import ViewletBase
from plone.app.uuid.utils import uuidToObject
from plone.base.interfaces import ISecuritySchema
from plone.base.interfaces import ISiteSchema
from plone.base.interfaces import ISiteSyndicationSettings
from plone.base.interfaces.syndication import IFeedSettings
from plone.base.utils import safe_bytes
from plone.formwidget.namedfile.converter import b64decode_file
from plone.memoize import ram
from plone.memoize import view
from plone.memoize.compress import xhtml_compress
from plone.registry.interfaces import IRegistry
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from typing import NoReturn
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.schema.interfaces import IVocabularyFactory


def get_language(context, request):
    portal_state = getMultiAdapter((context, request), name="plone_portal_state")
    return portal_state.language()


def render_cachekey(fun, self):
    # Include the name of the viewlet as the underlying cache key only
    # takes the module and function name into account, but not the class
    return "\n".join(
        [
            self.__name__,
            self.site_url,
            get_language(aq_inner(self.context), self.request),
        ]
    )


class FaviconViewlet(ViewletBase):
    _template = ViewPageTemplateFile("favicon.pt")
    mimetype: str
    favicon_path: str

    def init_favicon(self) -> NoReturn:
        registry = getUtility(IRegistry)
        settings: ISiteSchema = registry.forInterface(
            ISiteSchema,
            prefix="plone",
            check=False,
        )
        self.mimetype: str = getattr(
            settings, "site_favicon_mimetype", "image/vnd.microsoft.icon"
        )
        cachebust = ""
        if getattr(settings, "site_favicon", False):
            # The user has customized the favicon via the Site configlet.
            filename = b64decode_file(settings.site_favicon)[0]

            cachebust = "?name=" + filename
        # The filename is *always* /favicon.ico, irrespective of the content type,
        # because:
        #
        # 1. Browsers obey the content type over the extension.
        # 2. The actual serving view URL for the favicon is always /favicon.ico,
        #    and this name cannot be overridden from here into the view registered
        #    on CMFPlone, where the *actual* serving of the data takes place.
        # 3. Even if we could somehow override the view, there is no easy way to
        #    register in CMFPlone a different browser view for every icon file
        #    name the user may decide to upload.
        # 4. In many cases client applications just hit /favicon.ico irrespective
        #    of what the HTML says (remember that this specific view is only
        #    responsible for generating the metadata that lets the browser know
        #    where to find the favicon URL).
        #
        # However, to allow for users to change their favicons *and* bust their
        # proxy caches, we do use the favicon filename in the served favicon
        # URL.  This does not cover the case of RSS and podcast apps that access
        # /favicon.ico by custom instead of consulting the HTML, but at least
        # it covers pretty much every browser out there.
        self.favicon_path: str = f"{self.navigation_root_url}/favicon.ico{cachebust}"

    def render(self) -> ViewPageTemplateFile:
        self.init_favicon()
        return xhtml_compress(self._template())


class SearchViewlet(ViewletBase):
    _template = ViewPageTemplateFile("search.pt")

    @ram.cache(render_cachekey)
    def render(self):
        return xhtml_compress(self._template())


class AuthorViewlet(ViewletBase):
    _template = ViewPageTemplateFile("author.pt")

    def update(self):
        super().update()
        self.tools = getMultiAdapter((self.context, self.request), name="plone_tools")

    def show(self):
        anonymous = self.portal_state.anonymous()
        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            ISecuritySchema,
            prefix="plone",
        )
        return not anonymous or settings.allow_anon_views_about

    def render(self):
        if self.show():
            return self._template()
        return ""


class RSSViewlet(ViewletBase):
    def getRssLinks(self, obj):
        settings = IFeedSettings(obj, None)
        if settings is None:
            return []
        factory = getUtility(
            IVocabularyFactory, "plone.app.vocabularies.SyndicationFeedTypes"
        )
        vocabulary = factory(self.context)
        urls = []
        for typ in settings.feed_types:
            try:
                term = vocabulary.getTerm(typ)
            except LookupError:
                continue

            urls.append(
                {
                    "title": f"{obj.Title()} - {safe_bytes(term.title)}",
                    "url": obj.absolute_url() + "/" + term.value,
                }
            )
        return urls

    def update(self):
        super().update()
        self.rsslinks = []
        portal = self.portal_state.portal()
        util = getMultiAdapter((self.context, self.request), name="syndication-util")
        context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        if context_state.is_portal_root():
            if util.site_enabled():
                registry = getUtility(IRegistry)
                try:
                    settings = registry.forInterface(ISiteSyndicationSettings)
                except KeyError:
                    return
                if settings.site_rss_items:
                    for uid in settings.site_rss_items:
                        if not uid:
                            continue
                        try:
                            obj = uuidToObject(uid)
                        except Unauthorized:
                            # Do not break if we do not have enough permission
                            # to access the object
                            obj = None
                        if obj is None and uid[0] == "/":
                            obj = portal.restrictedTraverse(uid.lstrip("/"), None)
                        if obj is not None:
                            self.rsslinks.extend(self.getRssLinks(obj))
                self.rsslinks.extend(self.getRssLinks(portal))
        else:
            if util.context_enabled():
                self.rsslinks.extend(self.getRssLinks(self.context))

    index = ViewPageTemplateFile("rsslink.pt")


class CanonicalURL(ViewletBase):
    """Defines a canonical link relation viewlet to be displayed across the
    site. A canonical page is the preferred version of a set of pages with
    highly similar content. For more information, see:
    https://tools.ietf.org/html/rfc6596
    https://support.google.com/webmasters/answer/139394?hl=en
    """

    @view.memoize
    def render(self):
        context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        canonical_url = context_state.canonical_object_url()
        return '    <link rel="canonical" href="%s" />' % canonical_url



from AccessControl import getSecurityManager
from plone.app.layout.globals.interfaces import IBodyClassAdapter
from plone.app.layout.globals.interfaces import ILayoutPolicy
from plone.app.layout.globals.interfaces import IViewView
from plone.base.interfaces.controlpanel import ILinkSchema
from plone.base.interfaces.controlpanel import ISiteSchema
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.memoize.view import memoize
from plone.portlets.interfaces import IPortletManager
from plone.portlets.interfaces import IPortletManagerRenderer
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from Products.Five.browser.metaconfigure import ViewMixinForTemplates
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.browserpage.viewpagetemplatefile import (
    ViewPageTemplateFile as ZopeViewPageTemplateFile,
)
from zope.component import adapter
from zope.component import getAdapters
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.component import queryUtility
from zope.interface import alsoProvides
from zope.interface import implementer
from zope.interface import Interface
from zope.publisher.browser import BrowserView

import json


TEMPLATE_CLASSES = (
    ViewPageTemplateFile,
    ZopeViewPageTemplateFile,
    ViewMixinForTemplates,
)


@implementer(ILayoutPolicy)
class LayoutPolicy(BrowserView):
    """A view that gives access to various layout related functions."""

    @property
    @memoize
    def _context_state(self):
        return getMultiAdapter((self.context, self.request), name="plone_context_state")

    def mark_view(self, view):
        """Adds a marker interface to the view if it is "the" view for the
        context May only be called from a template.
        """
        if not view:
            return
        if self._context_state.is_view_template() and not IViewView.providedBy(view):
            alsoProvides(view, IViewView)

    def hide_columns(self, column_left, column_right):
        """Returns a CSS class matching the current column status."""
        if not column_right and not column_left:
            return "visualColumnHideOneTwo"
        if column_right and not column_left:
            return "visualColumnHideOne"
        if not column_right and column_left:
            return "visualColumnHideTwo"
        return "visualColumnHideNone"

    def have_portlets(self, manager_name, view=None):
        """Determine whether a column should be shown. The left column is
        called plone.leftcolumn; the right column is called plone.rightcolumn.
        """
        force_disable = self.request.get("disable_" + manager_name, None)
        if force_disable is not None:
            return not bool(force_disable)

        context = self.context
        if view is None:
            view = self

        manager = queryUtility(IPortletManager, name=manager_name)
        if manager is None:
            return False

        renderer = queryMultiAdapter(
            (context, self.request, view, manager), IPortletManagerRenderer
        )
        if renderer is None:
            renderer = getMultiAdapter(
                (context, self.request, self, manager), IPortletManagerRenderer
            )

        return renderer.visible

    def _image_visibility(self, name):
        """check if image {name} is visible with current settings and user"""
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        visibility = getattr(settings, f"{name}_visibility")
        if visibility == "enabled":
            return True
        if visibility != "authenticated":
            return False
        user = getSecurityManager().getUser()
        return user is not None and user.getUserName() != "Anonymous User"

    @memoize
    def icons_visible(self):
        """Returns True if icons should be shown or False otherwise."""
        return self._image_visibility("icon")

    @memoize
    def thumb_visible(self):
        """Returns True if thumbs should be shown or False otherwise."""
        return self._image_visibility("thumb")

    def _toolbar_classes(self):
        """current toolbar controlling classes"""
        if not self._context_state.is_toolbar_visible():
            return set()

        toolbar_classes = set()
        registry = getUtility(IRegistry)
        site_settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        try:
            left = site_settings.toolbar_position == "side"
        except KeyError:
            left = True
        if left:
            toolbar_classes.add("plone-toolbar-left")
        else:
            toolbar_classes.add("plone-toolbar-top")
        try:
            toolbar_state = {}
            toolbar_state_cookie = self.request.cookies.get("plone-toolbar")
            if toolbar_state_cookie:
                toolbar_state = json.loads(toolbar_state_cookie)
            if toolbar_state.get("expanded", True):
                toolbar_classes.add("plone-toolbar-expanded")
                if left:
                    toolbar_classes.add("plone-toolbar-left-expanded")
                else:
                    toolbar_classes.add("plone-toolbar-top-expanded")
            else:
                if left:
                    toolbar_classes.add("plone-toolbar-left-default")
                else:
                    toolbar_classes.add("plone-toolbar-top-default")
        except Exception:
            pass
        return toolbar_classes

    def _template_name(self, template, view):
        """Name of template.

        Sometimes it is best to take this from the template, sometimes the view.
        Note that neither template nor view are always available.
        For example, mosaic passes a view but not a template.
        """
        view_name = view.__name__ if view else ""
        if isinstance(template, TEMPLATE_CLASSES):
            # Browser view.  Take the view name.
            # But it this is 'index.html', it may be the error message view.
            # It is better to take the template name then.
            if view_name != "index.html":
                return view_name
        # template can be SimpleViewClass, which has no getId.
        # Probably caught above with isinstance(template, TEMPLATE_CLASSES),
        # but let's be careful.
        if template and hasattr(template, "getId"):
            template_name = template.getId()
            if template_name:
                return template_name
        return view_name

    def bodyClass(self, template, view):
        """
        Returns the CSS class to be used on the body tag.

        Included body classes:
        - template-{}: template name
        - portaltype-{}: portal type
        - site-{}: navigation root
        - section-{}: first section name
        - subsection-{}: subsection names until configured depth
        - icons-on: show icons
        - icons-off: hide icons
        - thumbs-on: show thumbnails
        - thumbs-off: hide thumbnails
        - frontend: user without privileges, no admin interfaces shown
        - viewpermission-{}: minimum permission needed to view context
        - userrole-anonymous: anonymous user
        - userrole-{}: user roles for current user
        - plone-toolbar-left: toolbar is shown on left side
        - plone-toolbar-top: toolbar is shown on top
        - plone-toolbar-expanded: toolbar is in expanded state
        - plone-toolbar-left-expanded: left toolbar is expanded
        - plone-toolbar-top-expanded: top toolbar is expanded
        - plone-toolbar-left-default: left toolbar is not expanded
        - plone-toolbar-top-default: top toolbar is not expanded
        - pat-markspeciallinks: mark special links is set
        """
        portal_state = getMultiAdapter(
            (self.context, self.request), name="plone_portal_state"
        )
        normalizer = queryUtility(IIDNormalizer)
        registry = getUtility(IRegistry)

        body_classes = self._toolbar_classes()

        # template class (required)
        template_name = self._template_name(template, view)
        if template_name:
            template_name = normalizer.normalize(template_name)
            body_classes.add("template-%s" % template_name)

        # portal type class (optional)
        portal_type = normalizer.normalize(self.context.portal_type)
        if portal_type:
            body_classes.add("portaltype-%s" % portal_type)

        # section class (optional)
        navroot = portal_state.navigation_root()
        body_classes.add("site-%s" % navroot.getId())

        contentPath = self.context.getPhysicalPath()[len(navroot.getPhysicalPath()) :]
        if contentPath:
            body_classes.add("section-%s" % contentPath[0])
            # skip first section since we already have that...
            if len(contentPath) > 1:
                depth = registry.get("plone.app.layout.globals.bodyClass.depth", 4)
                if depth > 1:
                    classes = ["subsection-%s" % contentPath[1]]
                    for section in contentPath[2:depth]:
                        classes.append("-".join([classes[-1], section]))
                    body_classes.update(classes)

        # class for hiding icons (optional)
        if self.icons_visible():
            body_classes.add("icons-on")
        else:
            body_classes.add("icons-off")

        # class for hiding thumbs (optional)
        if self.thumb_visible():
            body_classes.add("thumbs-on")
        else:
            body_classes.add("thumbs-off")

        # classes for column visibility: col-content, col-one, col-two:
        body_classes.add("col-content")
        if self.have_portlets("plone.leftcolumn", view):
            body_classes.add("col-one")
        if self.have_portlets("plone.rightcolumn", view):
            body_classes.add("col-two")

        # permissions required. Useful to theme frontend and backend
        # differently
        permissions = []
        if not getattr(view, "__ac_permissions__", tuple()):
            permissions = ["none"]
        for permission, roles in getattr(view, "__ac_permissions__", tuple()):
            permissions.append(normalizer.normalize(permission))
        if "none" in permissions or "view" in permissions:
            body_classes.add("frontend")
        for permission in permissions:
            body_classes.add("viewpermission-" + permission)

        # class for user roles
        membership = getToolByName(self.context, "portal_membership")
        if membership.isAnonymousUser():
            body_classes.add("userrole-anonymous")
        else:
            user = membership.getAuthenticatedMember()
            for role in user.getRolesInContext(self.context):
                body_classes.add("userrole-" + role.lower().replace(" ", "-"))

        # class for markspeciallinks pattern
        link_settings = registry.forInterface(ILinkSchema, prefix="plone", check=False)
        msl = link_settings.mark_special_links
        elonw = link_settings.external_links_open_new_window
        if msl or elonw:
            body_classes.add("pat-markspeciallinks")

        # Add externally defined extra body classes
        body_class_adapters = getAdapters(
            (self.context, self.request), IBodyClassAdapter
        )
        for name, body_class_adapter in body_class_adapters:
            try:
                extra_classes = body_class_adapter.get_classes(template, view) or []
            except TypeError:  # This adapter is implemented without arguments
                extra_classes = body_class_adapter.get_classes() or []
            if isinstance(extra_classes, str):
                extra_classes = extra_classes.split(" ")
            body_classes.update(extra_classes)

        return " ".join(sorted(body_classes))

    @property
    @memoize
    def is_ajax(self):
        """Check if the current request is an XHR request.

        Note: This is an unreliable way to detect AJAX requests.
        While many client-side libraries (like jQuery) add this request header
        automatically, the Fetch API does not. When using fetch, it is
        recommended to wrap it with a helper function that adds this header to
        each request.
        """
        return self.request.get("HTTP_X_REQUESTED_WITH") == "XMLHttpRequest"


@adapter(Interface, Interface)
@implementer(IBodyClassAdapter)
class DefaultBodyClasses:
    def __init__(self, context, request):
        self.context = context
        self.request = request

    def get_classes(self, template, view):
        """Default body classes adapter."""
        return []



from plone.app.layout.links.viewlets import RSSViewlet
from plone.app.layout.viewlets.tests.base import ViewletsTestCase
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.base.interfaces import ISiteSyndicationSettings
from plone.registry.interfaces import IRegistry
from zope.component import getUtility

import re


class TestRSSViewletView(ViewletsTestCase):
    def test_RSSViewlet(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory("Folder", "news")
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSyndicationSettings)
        if settings.allowed:
            # make sure syndication is disabled
            settings.allowed = False
        request = self.app.REQUEST
        viewlet = RSSViewlet(self.portal, request, None, None)
        viewlet.update()
        result = viewlet.render()
        self.assertEqual(result.strip(), "")
        settings.allowed = True
        settings.site_rss_items = (self.portal.news.UID(),)
        request = self.app.REQUEST
        viewlet = RSSViewlet(self.portal, request, None, None)
        viewlet.update()
        result = viewlet.render()
        self.assertFalse("<link" not in result)
        self.assertFalse("http://nohost/plone/atom.xml" not in result)
        self.assertFalse("http://nohost/plone/news/atom.xml" not in result)

    def test_RSSViewlet_with_private_objs(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory("Folder", "news")
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSyndicationSettings)
        self.assertTrue(settings.allowed)

        # Stream a private folder
        self.portal.news.invokeFactory("Collection", "aggregator")
        settings.site_rss_items = (self.portal.news.aggregator.UID(),)
        request = self.layer["request"]

        link_href_pattern = re.compile(r'<link href="(.*?)"')

        # Verify that anonymous users can't see the RSS feed
        # from the aggregator collection
        logout()
        viewlet = RSSViewlet(self.portal, request.clone(), None, None)
        viewlet.update()
        result = viewlet.render()

        self.assertSetEqual(
            {
                "http://nohost/plone/atom.xml",
                "http://nohost/plone/rss.xml",
                "http://nohost/plone/RSS",
            },
            {match.group(1) for match in link_href_pattern.finditer(result)},
        )

        login(self.portal, TEST_USER_NAME)
        viewlet = RSSViewlet(self.portal, request.clone(), None, None)
        viewlet.update()
        result = viewlet.render()

        # Verify that an authenticated user can see the RSS feed
        # from the aggregator collection
        self.assertSetEqual(
            {
                "http://nohost/plone/atom.xml",
                "http://nohost/plone/rss.xml",
                "http://nohost/plone/RSS",
                "http://nohost/plone/news/aggregator/atom.xml",
                "http://nohost/plone/news/aggregator/rss.xml",
                "http://nohost/plone/news/aggregator/RSS",
            },
            {match.group(1) for match in link_href_pattern.finditer(result)},
        )



from AccessControl import getSecurityManager
from Acquisition import aq_inner
from DateTime import DateTime
from plone.app.content.browser.interfaces import IFolderContentsView
from plone.app.layout.globals.interfaces import IViewView
from plone.app.layout.viewlets import ViewletBase
from plone.app.relationfield.behavior import IRelatedItems
from plone.base import PloneMessageFactory as _
from plone.base.interfaces import ISecuritySchema
from plone.base.interfaces import ISiteSchema
from plone.base.utils import base_hasattr
from plone.base.utils import logger
from plone.memoize.instance import memoize
from plone.memoize.view import memoize_contextless
from plone.protect.authenticator import createToken
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import _checkPermission
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFEditions.Permissions import AccessPreviousVersions
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from urllib.parse import urlencode
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.deprecation import deprecation


class DocumentActionsViewlet(ViewletBase):
    index = ViewPageTemplateFile("document_actions.pt")

    def update(self):
        super().update()

        self.context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        self.actions = self.context_state.actions("document_actions")


class DocumentBylineViewlet(ViewletBase):
    index = ViewPageTemplateFile("document_byline.pt")

    def update(self):
        super().update()
        self.anonymous = self.portal_state.anonymous()

    @property
    @deprecation.deprecate(
        "The context_state property is unused and will be removed in Plone 7"
    )
    def context_state(self):
        return getMultiAdapter((self.context, self.request), name="plone_context_state")

    @property
    @deprecation.deprecate(
        "The has_pam property is unused and will be removed in Plone 7"
    )
    def has_pam(self):
        return True

    @property
    @memoize_contextless
    def portal_membership(self):
        return getToolByName(self.context, "portal_membership")

    def show(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            ISiteSchema,
            prefix="plone",
            check=False,
        )
        return not self.anonymous or settings.display_publication_date_in_byline

    def show_about(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            ISecuritySchema,
            prefix="plone",
        )
        return not self.anonymous or settings.allow_anon_views_about

    @deprecation.deprecate(
        "The creator method is unused and will be removed in Plone 7"
    )
    def creator(self):
        return self.context.Creator()

    @deprecation.deprecate("The author method is unused and will be removed in Plone 7")
    def author(self):
        membership = getToolByName(self.context, "portal_membership")
        return membership.getMemberInfo(self.creator())

    @deprecation.deprecate(
        "The authorname method is unused and will be removed in Plone 7"
    )
    def authorname(self):
        author = self.author()
        return author and author["fullname"] or self.creator()

    @memoize
    def get_member_info(self, user_id):
        return self.portal_membership.getMemberInfo(user_id)

    def get_url_path(self, user_id):
        if self.get_member_info(user_id) is None:
            return ""
        if "/" in user_id:
            qs = urlencode({"author": user_id})
            return f"author/?{qs}"
        return f"author/{user_id}"

    def get_fullname(self, user_id):
        info = self.get_member_info(user_id)
        if info is None:
            return user_id
        return info.get("fullname") or user_id

    def show_modification_date(self):
        return not self.context.effective_date or (
            self.context.effective_date.Date() < self.context.modification_date.Date()
        )

    def isExpired(self):
        if base_hasattr(self.context, "expires"):
            return self.context.expires().isPast()
        return False

    @deprecation.deprecate(
        "The toLocalizedTime method is unused and will be removed in Plone 7. "
        "Use the @@plone.toLocalizedTime helper instead."
    )
    def toLocalizedTime(self, time, long_format=None, time_only=None):
        """Convert time to localized time"""
        util = getToolByName(self.context, "translation_service")
        return util.ulocalized_time(
            time, long_format, time_only, self.context, domain="plonelocales"
        )

    def pub_date(self):
        """Return object effective date.

        Return None if publication date is switched off in global site settings
        or if Effective Date is not set on object.
        """
        # check if we are allowed to display publication date
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)

        if not settings.display_publication_date_in_byline:
            return None

        # check if we have Effective Date set
        date = self.context.EffectiveDate()
        if not date or date == "None":
            return None

        return DateTime(date)

    @deprecation.deprecate(
        "The get_translations method is unused and will be removed in Plone 7"
    )
    def get_translations(self):
        from plone.app.multilingual.browser.vocabularies import translated_languages
        from plone.app.multilingual.interfaces import ITranslatable
        from plone.app.multilingual.interfaces import ITranslationManager

        cts = []
        if ITranslatable.providedBy(self.context):
            t_langs = translated_languages(self.context)
            context_translations = ITranslationManager(self.context).get_translations()
            for lang in t_langs:
                cts.append(
                    dict(
                        lang_native=lang.title,
                        url=context_translations[lang.value].absolute_url(),
                    )
                )
        return cts


class HistoryByLineView(BrowserView):
    """DocumentByLine information for content history view"""

    index = ViewPageTemplateFile("history_view.pt")

    def update(self):
        context = self.context
        self.portal_state = getMultiAdapter(
            (context, self.request), name="plone_portal_state"
        )
        self.context_state = getMultiAdapter(
            (self.context, self.request), name="plone_context_state"
        )
        self.anonymous = self.portal_state.anonymous()

    def __call__(self):
        self.update()

        return self.index()

    @property
    @deprecation.deprecate(
        "The has_pam property is unused and will be removed in Plone 7"
    )
    def has_pam(self):
        return True

    def show(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            ISecuritySchema,
            prefix="plone",
        )
        return not self.anonymous or settings.allow_anon_views_about

    def show_history(self):
        has_access_preview_versions_permission = _checkPermission(
            "CMFEditions: Access previous versions", self.context
        )
        if not has_access_preview_versions_permission:
            return False
        if IViewView.providedBy(self.__parent__):
            return True
        if IFolderContentsView.providedBy(self.__parent__):
            return True
        return False

    def locked_icon(self):
        if not getSecurityManager().checkPermission(
            "Modify portal content", self.context
        ):
            return ""

        locked = False
        lock_info = queryMultiAdapter(
            (self.context, self.request), name="plone_lock_info"
        )
        if lock_info is not None:
            locked = lock_info.is_locked()
        else:
            context = aq_inner(self.context)
            lockable = getattr(context.aq_explicit, "wl_isLocked", None) is not None
            locked = lockable and context.wl_isLocked()

        if not locked:
            return ""

        portal = self.portal_state.portal()
        icon = portal.restrictedTraverse("lock_icon.png")
        return icon.tag(title="Locked")

    def creator(self):
        return self.context.Creator()

    def author(self):
        membership = getToolByName(self.context, "portal_membership")
        return membership.getMemberInfo(self.creator())

    def authorname(self):
        author = self.author()
        return author and author["fullname"] or self.creator()

    def isExpired(self):
        if base_hasattr(self.context, "expires"):
            return self.context.expires().isPast()
        return False

    @deprecation.deprecate(
        "The toLocalizedTime method is unused and will be removed in Plone 7. "
        "Use the @@plone.toLocalizedTime helper instead."
    )
    def toLocalizedTime(self, time, long_format=None, time_only=None):
        """Convert time to localized time"""
        util = getToolByName(self.context, "translation_service")
        return util.ulocalized_time(
            time, long_format, time_only, self.context, domain="plonelocales"
        )

    def pub_date(self):
        """Return object effective date.

        Return None if publication date is switched off in global site settings
        or if Effective Date is not set on object.
        """
        # check if we are allowed to display publication date
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)

        if not settings.display_publication_date_in_byline:
            return None

        # check if we have Effective Date set
        date = self.context.EffectiveDate()
        if not date or date == "None":
            return None

        return DateTime(date)

    @deprecation.deprecate(
        "The get_translations method is unused and will be removed in Plone 7"
    )
    def get_translations(self):
        from plone.app.multilingual.browser.vocabularies import translated_languages
        from plone.app.multilingual.interfaces import ITranslatable
        from plone.app.multilingual.interfaces import ITranslationManager

        cts = []
        if ITranslatable.providedBy(self.context):
            t_langs = translated_languages(self.context)
            context_translations = ITranslationManager(self.context).get_translations()
            for lang in t_langs:
                cts.append(
                    dict(
                        lang_native=lang.title,
                        url=context_translations[lang.value].absolute_url(),
                    )
                )

        return cts


class ContentRelatedItems(ViewletBase):
    index = ViewPageTemplateFile("document_relateditems.pt")

    def related_items(self):
        if not IRelatedItems.providedBy(self.context):
            return ()
        related = aq_inner(self.context).relatedItems
        if not related:
            return ()
        return self.related2brains(related)

    def related2brains(self, related):
        """Return a list of brains based on a list of relations. Will filter
        relations if the user has no permission to access the content.

        :param related: related items
        :type related: list of relations
        :return: list of catalog brains
        """
        catalog = getToolByName(self.context, "portal_catalog")
        brains = []
        for r in related:
            path = r.to_path
            if path is None:
                # Item was deleted.  The related item should have been cleaned
                # up, but apparently this does not happen.
                continue
            # the query will return an empty list if the user
            # has no permission to see the target object
            brains.extend(catalog(path=dict(query=path, depth=0)))
        return brains


class WorkflowHistoryViewlet(ViewletBase):
    index = ViewPageTemplateFile("review_history.pt")

    @memoize
    def getUserInfo(self, userid):
        actor = dict(fullname=userid)
        mt = getToolByName(self.context, "portal_membership")
        info = mt.getMemberInfo(userid)
        if info is None:
            return dict(actor_home="", actor=actor)

        fullname = info.get("fullname", None)
        if fullname:
            actor["fullname"] = fullname

        return dict(actor=actor, actor_home=f"{self.site_url}/author/{userid}")

    def workflowHistory(self, complete=True):
        """Return workflow history of this context.

        Taken from plone_scripts/getWorkflowHistory.py
        """
        context = aq_inner(self.context)
        # check if the current user has the proper permissions
        if not (
            _checkPermission("Request review", context)
            or _checkPermission("Review portal content", context)
        ):
            return []

        workflow = getToolByName(context, "portal_workflow")
        review_history = []

        try:
            # Get total history.
            # Note: expected variables like 'action' may not exist:
            # the workflow may have started out without variables.
            review_history = workflow.getInfoFor(context, "review_history")

            if not complete:
                # filter out automatic transitions.
                review_history = [r for r in review_history if r.get("action")]
            else:
                review_history = list(review_history)

            portal_type = context.portal_type
            anon = _("label_anonymous_user", default="Anonymous User")
            for r in review_history:
                r["type"] = "workflow"

                # Get transition title.
                transition_title = ""
                action = r.get("action")
                if action:
                    transition_title = workflow.getTitleForTransitionOnType(
                        action, portal_type
                    )
                if not transition_title:
                    transition_title = _("Create")
                r["transition_title"] = transition_title

                # Get state title.
                r["state_title"] = workflow.getTitleForStateOnType(
                    r.get("review_state", ""), portal_type
                )

                # Get actor.
                actorid = r.get("actor")
                r["actorid"] = actorid
                if actorid is None:
                    # action performed by an anonymous user, or unknown
                    r["actor"] = {"username": anon, "fullname": anon}
                    r["actor_home"] = ""
                else:
                    r.update(self.getUserInfo(actorid))
            review_history.reverse()

        except WorkflowException:
            logger.debug(
                "plone.app.layout.viewlets.content: %s has no associated workflow",
                context.absolute_url(),
            )

        return review_history


class ContentHistoryViewlet(WorkflowHistoryViewlet):
    index = ViewPageTemplateFile("content_history.pt")

    def revisionHistory(self):
        context = aq_inner(self.context)
        if not _checkPermission(AccessPreviousVersions, context):
            return []

        rt = getToolByName(context, "portal_repository", None)
        if rt is None or not rt.isVersionable(context):
            return []

        context_url = context.absolute_url()
        history = rt.getHistoryMetadata(context)
        portal_diff = getToolByName(context, "portal_diff", None)
        can_diff = (
            portal_diff is not None
            and len(portal_diff.getDiffForPortalType(context.portal_type)) > 0
        )
        can_revert = _checkPermission(
            "CMFEditions: Revert to previous versions", context
        )

        def morphVersionDataToHistoryFormat(vdata, version_id):
            meta = vdata["metadata"]["sys_metadata"]
            userid = meta["principal"]
            token = createToken()
            preview_url = (
                "%s/versions_history_form?version_id=%s&_authenticator=%s#version_preview"
                % (context_url, version_id, token)  # noqa
            )
            info = dict(
                type="versioning",
                action=_("Edited"),
                transition_title=_("Edited"),
                actorid=userid,
                time=meta["timestamp"],
                comments=meta["comment"],
                version_id=version_id,
                preview_url=preview_url,
            )
            if can_diff:
                if version_id > 0:
                    info["diff_previous_url"] = (
                        "{}/@@history?one={}&two={}&_authenticator={}".format(
                            context_url,
                            version_id,
                            version_id - 1,
                            token,
                        )
                    )
                if not rt.isUpToDate(context, version_id):
                    info["diff_current_url"] = (
                        "{}/@@history?one=current&two={}&_authenticator={}".format(
                            context_url,
                            version_id,
                            token,
                        )
                    )
            if can_revert:
                info["revert_url"] = "%s/revertversion" % context_url
            else:
                info["revert_url"] = None
            info.update(self.getUserInfo(userid))
            return info

        # History may be an empty list
        if not history:
            return history

        version_history = []
        retrieve = history.retrieve
        getId = history.getVersionId
        # Count backwards from most recent to least recent
        for i in range(history.getLength(countPurged=False) - 1, -1, -1):
            version_history.append(
                morphVersionDataToHistoryFormat(
                    retrieve(i, countPurged=False), getId(i, countPurged=False)
                )
            )

        return version_history

    def fullHistory(self):
        history = self.workflowHistory() + self.revisionHistory()
        if len(history) == 0:
            return None
        history.sort(key=lambda x: x.get("time", 0.0), reverse=True)
        return history

    @deprecation.deprecate(
        "The toLocalizedTime method is unused and will be removed in Plone 7. "
        "Use the @@plone.toLocalizedTime helper instead."
    )
    def toLocalizedTime(self, time, long_format=None, time_only=None):
        """Convert time to localized time"""
        util = getToolByName(self.context, "translation_service")
        return util.ulocalized_time(
            time, long_format, time_only, self.context, domain="plonelocales"
        )


class ContentHistoryView(ContentHistoryViewlet):
    def __init__(self, context, request):
        super().__init__(context, request, None, None)
        self.update()

    def __call__(self):
        return self.index()



from plone.base.interfaces import ISiteSchema
from plone.registry.interfaces import IRegistry
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.component import getUtility
from zope.interface import implementer
from zope.viewlet.interfaces import IViewlet


UNWANTED_TAGS = ["base", "title"]


@implementer(IViewlet)
class AnalyticsViewlet(BrowserView):
    render = ViewPageTemplateFile("view.pt")
    record_name = "webstats_js"

    def __init__(self, context, request, view, manager):
        super().__init__(context, request)
        self.__parent__ = view
        self.view = view
        self.manager = manager

    @property
    def webstats_js(self):
        registry = getUtility(IRegistry)
        site_settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        return getattr(site_settings, self.record_name, "")

    def update(self):
        """The viewlet manager _updateViewlets requires this method"""
        pass


@implementer(IViewlet)
class AnalyticsHeadViewlet(AnalyticsViewlet):
    render = ViewPageTemplateFile("view_head.pt")
    record_name = "webstats_head_js"



from zope.interface import Interface


class IInterfaceInformation(Interface):
    """A view that gives information about interfaces provided by its context"""

    def provides(dotted_name):
        """Given an interface dotted name, determine if the context provides
        this interface.
        """

    def class_provides(dotted_name):
        """Given an interface dotted name, determine if the context's class
        provides this interface.
        """

    def names_and_descriptions(dotted_name, all):
        """Returns a list of pairs (name, description) for a given
        interface"""

    def get_interfaces():
        """Returns the list of interfaces which are implemented by the object"""

    def get_base_interface():
        """Returns all base interfaces of an object but no direct interfaces

        Base interfaces are the interfaces which are the super interfaces of
        the direct interfaces
        """

    def get_interface_informations(self, iface):
        """Gets all useful information from an iface

        * name
        * dotted name
        * trimmed doc string
        * base interfaces
        * methods with signature and trimmed doc string
        * attributes with trimemd doc string
        """


class ILayoutPolicy(Interface):
    """A view that gives access to various layout related functions."""

    def mark_view(view):
        """Adds a marker interface to the view if it is "the" view for the
        context May only be called from a template.
        """

    def hide_columns(column_left, column_right):
        """Returns a CSS class matching the current column status."""

    def have_portlets(manager_name, view=None):
        """Determine whether a column should be shown. The left column is
        called plone.leftcolumn; the right column is called plone.rightcolumn.
        """

    def icons_visible():
        """Returns True if icons should be shown or False otherwise."""

    def bodyClass(template, view):
        """Returns the CSS class to be used on the body tag."""


class ITools(Interface):
    """A view that gives access to common tools"""

    def actions():
        """The portal_actions tool"""

    def catalog():
        """The portal_catalog tool"""

    def membership():
        """The portal_membership tool"""

    def types():
        """The portal_types tool"""

    def url():
        """The portal_url tool"""

    def workflow():
        """The portal_workflow tool"""


class IPortalState(Interface):
    """A view that gives access to the current state of the portal"""

    def portal():
        """The portal object"""

    def portal_title():
        """The title of the portal object"""

    def portal_url():
        """The URL of the portal object"""

    def navigation_root():
        """The navigation root object"""

    def navigation_root_title():
        """The title of the navigation root object"""

    def navigation_root_path():
        """path of the navigation root"""

    def navigation_root_url():
        """The URL of the navigation root"""

    def default_language():
        """The default language in the portal"""

    def language():
        """The current language"""

    def locale():
        """Get the current locale"""

    def is_rtl():
        """Whether or not the portal is being viewed in an RTL language"""

    def member():
        """The current authenticated member"""

    def anonymous():
        """Whether or not the current member is Anonymous"""

    def friendly_types():
        """Get a list of portal types considered "end user" types"""


class IContextState(Interface):
    """A view that gives access to the state of the current context"""

    def current_page_url():
        """The URL to the current page, including template and query string."""

    def current_base_url():
        """The current "actual" URL from the request, excluding the query
        string.
        """

    def canonical_object():
        """The current "canonical" object.

        That is, the current object unless this object is the default page
        in its folder, in which case the folder is returned.
        """

    def canonical_object_url():
        """The URL to the current "canonical" object.

        That is, the current object unless this object is the default page
        in its folder, in which case the folder is returned.
        """

    def view_url():
        """URL to use for viewing

        Files and Images get downloaded when they are directly
        called, instead of with /view appended.  We want to avoid that.
        """

    def view_template_id():
        """The id of the view template of the context"""

    def is_view_template():
        """Return True if the current URL (in the request) refers to the
        standard "view" of the context (i.e. the "view" tab).
        """

    def object_url():
        """The URL of the current object"""

    def object_title():
        """The prettified title of the current object"""

    def workflow_state():
        """The workflow state of the current object"""

    def parent():
        """The direct parent of the current object"""

    def folder():
        """The current canonical folder"""

    def is_folderish():
        """True if this is a folderish object, structural or not"""

    def is_structural_folder():
        """True if this is a structural folder"""

    def is_default_page():
        """True if this is the default page of its folder"""

    def is_portal_root():
        """True if this is the portal or the default page in the portal"""

    def is_editable():
        """Whether or not the current object is editable"""

    def is_locked():
        """Whether or not the current object is locked"""

    def is_toolbar_visible():
        """Whether toolbar is visible or not in the actual context"""

    def actions(category):
        """The filtered actions in the context. You can restrict the actions
        to just one category.
        """

    def portlet_assignable():
        """Whether or not the context is capable of having locally assigned
        portlets.
        """


class IViewView(Interface):
    """Marker interface which specifies that the current view is, in fact,
    a canonical "view" of the object, e.g. what may go on the "view" tab.
    """


class IBodyClassAdapter(Interface):
    """Adapter interface for retrieving extra body classes."""



from zope.deferredimport import deprecated


deprecated(
    "Import as get_navigation_root from plone.base.navigationroot instead (will be removed in Plone 7)",
    getNavigationRoot="plone.base.navigationroot:get_navigation_root",
)
deprecated(
    "Import as get_navigation_root_object from plone.base.navigationroot instead (will be removed in Plone 7)",
    getNavigationRootObject="plone.base.navigationroot:get_navigation_root_object",
)



from plone.app.layout.viewlets.social import SocialTagsViewlet
from plone.app.layout.viewlets.tests.base import ViewletsTestCase
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.base.interfaces import ISocialMediaSchema
from plone.registry.interfaces import IRegistry
from zope.annotation.interfaces import IAnnotations
from zope.component import getUtility


class TestSocialViewlet(ViewletsTestCase):
    """Test the content views viewlet."""

    def setUp(self):
        super().setUp()
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.folder.invokeFactory("News Item", "news-item", title="News Item")
        self.news = self.folder["news-item"]
        logout()

    def _tagFound(self, tags, attr, name=None, value=None):
        for meta in tags:
            if attr in meta:
                if name is None:
                    # only checking for existence
                    return True
                if meta[attr] == name:
                    if value is None:
                        # only checking for existence
                        return True
                    return meta["content"] == value
        return False

    def tagFound(self, viewlet, attr, name=None, value=None):
        return self._tagFound(viewlet.tags, attr, name=name, value=value)

    def bodyTagFound(self, viewlet, attr, name=None, value=None):
        return self._tagFound(viewlet.body_tags, attr, name=name, value=value)

    def testBasicTags(self):
        viewlet = SocialTagsViewlet(self.folder, self.app.REQUEST, None)
        viewlet.update()
        description = self.folder.Description()
        folder_url = self.folder.absolute_url()
        # Twitter
        self.assertTrue(self.tagFound(viewlet, "name", "twitter:card", "summary"))
        # OpenGraph/Facebook
        self.assertTrue(
            self.tagFound(
                viewlet, "property", "og:site_name", viewlet.site_title_setting
            )
        )
        self.assertTrue(
            self.tagFound(viewlet, "property", "og:title", viewlet.page_title)
        )
        self.assertTrue(
            self.tagFound(viewlet, "property", "og:description", description)
        )
        self.assertTrue(self.tagFound(viewlet, "property", "og:url", folder_url))
        # No schema.org itemprops
        self.assertFalse(self.tagFound(viewlet, "itemprop"))

    def testBasicItemProps(self):
        viewlet = SocialTagsViewlet(self.folder, self.app.REQUEST, None)
        viewlet.update()
        description = self.folder.Description()
        folder_url = self.folder.absolute_url()
        # No Twitter
        self.assertFalse(self.bodyTagFound(viewlet, "name"))
        # No OpenGraph/Facebook
        self.assertFalse(self.bodyTagFound(viewlet, "property"))
        # schema.org itemprops
        self.assertTrue(
            self.bodyTagFound(viewlet, "itemprop", "name", viewlet.page_title)
        )
        self.assertTrue(
            self.bodyTagFound(viewlet, "itemprop", "description", description)
        )
        self.assertTrue(self.bodyTagFound(viewlet, "itemprop", "url", folder_url))

    def testDisabled(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            ISocialMediaSchema, prefix="plone", check=False
        )
        settings.share_social_data = False
        viewlet = SocialTagsViewlet(self.folder, self.app.REQUEST, None)
        viewlet.update()
        self.assertEqual(len(viewlet.tags), 0)

    def testDisabledForLoggedUser(self):
        login(self.portal, TEST_USER_NAME)
        viewlet = SocialTagsViewlet(self.folder, self.app.REQUEST, None)
        viewlet.update()
        self.assertEqual(len(viewlet.tags), 0)
        # clear cache to prevent memoize
        cache = IAnnotations(self.app.REQUEST)
        key = "plone.memoize"
        cache[key] = {}
        logout()
        viewlet.update()
        self.assertTrue(len(viewlet.tags) > 1)

    def testIncludeSocialSettings(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(
            ISocialMediaSchema, prefix="plone", check=False
        )
        settings.twitter_username = "foobar"
        settings.facebook_app_id = "foobar"
        settings.facebook_username = "foobar"
        viewlet = SocialTagsViewlet(self.folder, self.app.REQUEST, None)
        viewlet.update()
        self.assertTrue(self.tagFound(viewlet, "name", "twitter:site", "@foobar"))
        self.assertTrue(self.tagFound(viewlet, "property", "fb:app_id", "foobar"))
        self.assertTrue(
            self.tagFound(
                viewlet,
                "property",
                "og:article:publisher",
                "https://www.facebook.com/foobar",
            )
        )

    def testLogo(self):
        viewlet = SocialTagsViewlet(self.news, self.app.REQUEST, None)
        viewlet.update()
        self.assertTrue(
            self.tagFound(
                viewlet,
                "property",
                "og:image",
                "http://nohost/plone/++resource++plone-logo.svg",
            )
        )
        self.assertFalse(self.tagFound(viewlet, "itemprop"))
        self.assertTrue(
            self.bodyTagFound(
                viewlet,
                "itemprop",
                "image",
                "http://nohost/plone/++resource++plone-logo.svg",
            )
        )



from zope.interface import Attribute
from zope.interface import Interface


class IContentIcon(Interface):
    """An icon for a piece of content"""

    width = Attribute("The width of the icon (required).")

    height = Attribute("Returns the height of the icon (required).")

    url = Attribute(
        "The absolute url of the icon. This should be "
        "None if no icon should be rendered."
    )

    description = Attribute(
        "The description used for the alt attribute. "
        "Should be at least an empty string."
    )

    title = Attribute(
        "The content of the title attribute. Should be None " "if the title is empty."
    )

    def html_tag():
        """Return a HTML string that is the tag for rendering this icon."""



from DateTime import DateTime
from gzip import GzipFile
from io import BytesIO
from plone.app.layout.testing import INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.base.interfaces import INavigationRoot
from plone.base.interfaces import ISearchSchema
from plone.base.interfaces import ISiteSchema
from plone.base.utils import safe_text
from plone.registry.interfaces import IRegistry
from Products.CMFCore.utils import getToolByName
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.interface import alsoProvides
from zope.publisher.interfaces import INotFound

import unittest


class SiteMapTestCase(unittest.TestCase):
    """base test case with convenience methods for all sitemap tests"""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        registry = getUtility(IRegistry)
        self.site_settings = registry.forInterface(ISiteSchema, prefix="plone")
        self.site_settings.enable_sitemap = True
        self.sitemap = getMultiAdapter(
            (self.portal, self.portal.REQUEST), name="sitemap.xml.gz"
        )
        self.wftool = getToolByName(self.portal, "portal_workflow")

        # we need to explicitly set a workflow cause we can't rely on the
        # test environment.
        # `instance test -m plone.app.layout`:
        # wftool._default_chain == 'simple_publication_workflow'
        # `instance test -m plone.app`:
        # wftool._default_chain == 'plone_workflow'
        self.wftool.setChainForPortalTypes(["Document"], "simple_publication_workflow")

        # setup private content that isn't accessible for anonymous
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory(id="private", type_name="Document")
        private = self.portal.private
        self.assertTrue("private" == self.wftool.getInfoFor(private, "review_state"))

        # setup published content that is accessible for anonymous
        self.portal.invokeFactory(id="published", type_name="Document")
        published = self.portal.published
        self.wftool.doActionFor(published, "publish")
        self.assertTrue(
            "published" == self.wftool.getInfoFor(published, "review_state")
        )

        # setup pending content that isn't accessible for anonymous
        self.portal.invokeFactory(id="pending", type_name="Document")
        pending = self.portal.pending
        self.wftool.doActionFor(pending, "submit")
        self.assertTrue("pending" == self.wftool.getInfoFor(pending, "review_state"))
        logout()

    def uncompress(self, sitemapdata):
        sio = BytesIO(sitemapdata)
        unzipped = GzipFile(fileobj=sio)
        xml = unzipped.read()
        unzipped.close()
        return safe_text(xml)

    def test_disabled(self):
        """
        If the sitemap is disabled throws a 404 error.
        """
        self.site_settings.enable_sitemap = False
        try:
            self.sitemap()
        except Exception as e:
            # zope2 and repoze.zope2 use different publishers and raise
            # different exceptions. but both implement INotFound.
            self.assertTrue(INotFound.providedBy(e))
        else:
            self.fail("The disabled sitemap view has to raise NotFound!")

    def test_authenticated_before_anonymous(self):
        """
        Requests for the sitemap by authenticated users are not cached.
        anomymous users get a uncached sitemap that only contains content
        that they are supposed to see.
        """

        # first round as an authenticated (manager)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.REQUEST["page"] = 1
        xml = self.uncompress(self.sitemap())
        self.assertTrue("<loc>http://nohost/plone/private</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/pending</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/published</loc>" in xml)

        # second round as anonymous
        logout()
        self.portal.REQUEST["page"] = 1
        xml = self.uncompress(self.sitemap())
        self.assertFalse("<loc>http://nohost/plone/private</loc>" in xml)
        self.assertFalse("<loc>http://nohost/plone/pending</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/published</loc>" in xml)

    def test_anonymous_before_authenticated(self):
        """
        Requests for the sitemap by anonymous users are cached.
        authenticated users get a uncached sitemap. Test that the cached
        Sitemap is not delivered to authenticated users.
        """

        # first round as anonymous
        self.portal.REQUEST["page"] = 1
        xml = self.uncompress(self.sitemap())
        self.assertFalse("<loc>http://nohost/plone/private</loc>" in xml)
        self.assertFalse("<loc>http://nohost/plone/pending</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/published</loc>" in xml)

        # second round as an authenticated (manager)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.REQUEST["page"] = 1
        xml = self.uncompress(self.sitemap())
        self.assertTrue("<loc>http://nohost/plone/private</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/pending</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/published</loc>" in xml)

    def test_changed_catalog(self):
        """
        The sitemap is generated from the catalog. If the catalog changes, a
        new sitemap has to be generated.
        """
        self.portal.REQUEST["page"] = 1
        xml = self.uncompress(self.sitemap())
        self.assertFalse("<loc>http://nohost/plone/pending</loc>" in xml)

        # changing the workflow state
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        pending = self.portal.pending
        self.wftool.doActionFor(pending, "publish")
        logout()

        self.portal.REQUEST["page"] = 1
        xml = self.uncompress(self.sitemap())
        self.assertTrue("<loc>http://nohost/plone/pending</loc>" in xml)

        # removing content
        login(self.portal, TEST_USER_NAME)
        self.portal.manage_delObjects(
            [
                "published",
            ]
        )
        logout()

        self.portal.REQUEST["page"] = 1
        xml = self.uncompress(self.sitemap())
        self.assertFalse("<loc>http://nohost/plone/published</loc>" in xml)

    def test_navroot(self):
        """
        Sitemap generated from an INavigationRoot
        """
        # setup navroot content that is accessible for anonymous
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory(id="navroot", type_name="Folder")
        navroot = self.portal.navroot
        self.wftool.doActionFor(navroot, "publish")
        self.assertTrue("published" == self.wftool.getInfoFor(navroot, "review_state"))
        alsoProvides(navroot, INavigationRoot)
        navroot.invokeFactory(id="published", type_name="Document")
        published = navroot.published
        self.wftool.doActionFor(published, "publish")
        self.assertTrue(
            "published" == self.wftool.getInfoFor(published, "review_state")
        )
        logout()

        sitemap = getMultiAdapter(
            (self.portal.navroot, self.portal.REQUEST), name="sitemap.xml.gz"
        )
        self.portal.REQUEST["page"] = 1
        xml = self.uncompress(sitemap())
        self.assertFalse("<loc>http://nohost/plone/published</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/navroot</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/navroot/published</loc>" in xml)

    def test_types_not_searched(self):
        """
        Test that types_not_searched is respected
        """
        # Set News Items not to be searchable (more likely Images)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory(id="newsitem", type_name="News Item")
        newsitem = self.portal.newsitem
        self.wftool.doActionFor(newsitem, "publish")
        self.assertTrue("published" == self.wftool.getInfoFor(newsitem, "review_state"))
        registry = getUtility(IRegistry)
        search_settings = registry.forInterface(ISearchSchema, prefix="plone")
        search_settings.types_not_searched = ("News Item",)
        logout()

        self.portal.REQUEST["page"] = 1
        xml = self.uncompress(self.sitemap())
        self.assertFalse("<loc>http://nohost/plone/newsitem</loc>" in xml)

    def test_typesUseViewActionInListings(self):
        """
        Test that typesUseViewActionInListings is respected
        """
        # Set News Items not to be searchable (more likely Images)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory(id="newsitem", type_name="News Item")
        newsitem = self.portal.newsitem
        self.wftool.doActionFor(newsitem, "publish")
        self.assertTrue("published" == self.wftool.getInfoFor(newsitem, "review_state"))
        registry = getUtility(IRegistry)
        registry["plone.types_use_view_action_in_listings"] = ["News Item"]

        logout()

        self.portal.REQUEST["page"] = 1
        xml = self.uncompress(self.sitemap())
        self.assertTrue("<loc>http://nohost/plone/newsitem/view</loc>" in xml)

    def test_default_pages(self):
        """
        Default pages should show up at their parent's url with the greater of
        their or their parent's modification time.
        """

        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory(id="folder", type_name="Folder")
        folder = self.portal.folder
        folder.default_page = "default"
        self.wftool.doActionFor(folder, "publish")
        self.assertTrue("published" == self.wftool.getInfoFor(folder, "review_state"))

        folder.invokeFactory(id="default", type_name="Document")
        default = folder.default
        self.wftool.doActionFor(default, "publish")
        self.assertTrue("published" == self.wftool.getInfoFor(default, "review_state"))
        self.assertTrue(self.portal.plone_utils.isDefaultPage(default))
        default.modification_date = DateTime("2001-01-01")
        folder.modification_date = DateTime("2000-01-01")
        self.portal.portal_catalog.reindexObject(
            folder, idxs=["modified", "is_default_page", "effective"]
        )
        self.portal.portal_catalog.reindexObject(
            default, idxs=["modified", "is_default_page", "effective"]
        )
        self.portal.default_page = "published"
        self.portal.portal_catalog.reindexObject(
            self.portal.published, idxs=["modified", "is_default_page", "effective"]
        )
        logout()

        self.portal.REQUEST["page"] = 1
        xml = self.uncompress(self.sitemap())
        self.assertFalse("<loc>http://nohost/plone/folder/default</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/folder</loc>" in xml)
        self.assertTrue("<lastmod >2001-01-01T" in xml)
        self.assertTrue("<loc>http://nohost/plone</loc>" in xml)
        self.assertFalse("<loc>http://nohost/plone/published</loc>" in xml)

    def test_rendering_of_sitemap_index(self):
        """check that when calling without parameters,
        it renders a sitemap index file pointing to the specific files
        """
        # first round as an authenticated (manager)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)
        xml = self.uncompress(self.sitemap())

        self.assertTrue("<sitemapindex" in xml)
        self.assertTrue("<loc>http://nohost/plone/sitemap.xml.gz?page=1</loc>" in xml)
        self.assertTrue(
            "<loc>http://nohost/plone/sitemap.xml.gz?page=2</loc>" not in xml
        )

    def test_rendering_multiple_sitemaps(self):
        """test that when we have more than the batch size count items"""
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)

        # Generate 5000 items to have at least 2 sitemap URLs generated in the index file
        for count in range(5000):
            self.portal.invokeFactory(id=f"private-{count}", type_name="Document")

        xml = self.uncompress(self.sitemap())

        self.assertTrue("<sitemapindex" in xml)
        self.assertTrue("<loc>http://nohost/plone/sitemap.xml.gz?page=1</loc>" in xml)
        self.assertTrue("<loc>http://nohost/plone/sitemap.xml.gz?page=2</loc>" in xml)
        self.assertTrue(
            "<loc>http://nohost/plone/sitemap.xml.gz?page=3</loc>" not in xml
        )

    def test_first_page_has_batch_size_count_items(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        login(self.portal, TEST_USER_NAME)

        # Generate 5000 items to have at least 2 sitemap URLs generated in the index file
        for count in range(5000):
            self.portal.invokeFactory(id=f"private-{count}", type_name="Document")

        self.portal.REQUEST["page"] = 1

        xml = self.uncompress(self.sitemap())
        self.assertEqual(xml.count("<loc>"), 5000)

        # Now we try the second batch
        self.portal.REQUEST["page"] = 2

        xml = self.uncompress(self.sitemap())
        self.assertEqual(xml.count("<loc>"), 6)



from plone.app.layout.testing import INTEGRATION_TESTING
from zope.component import getMultiAdapter

import unittest


class TestAttackVector(unittest.TestCase):
    layer = INTEGRATION_TESTING

    def test_using_correct_template(self):
        """Ensure that confirm-action view uses the confirm.pt template from plone.app.layout."""
        portal = self.layer["portal"]
        request = self.layer["request"]
        view = getMultiAdapter((portal, request), name="confirm-action")

        self.assertIn("plone/app/layout/views/confirm.pt", view.index.filename)



from .interfaces import IInterfaceInformation
from Acquisition import aq_base
from plone.memoize.view import memoize
from Products.Five.browser import BrowserView
from zope.dottedname.resolve import resolve
from zope.interface import implementer
from zope.interface import Interface
from zope.interface import providedBy
from zope.interface.interfaces import IMethod


def resolveInterface(dotted_name):
    klass = resolve(dotted_name)
    if not issubclass(klass, Interface):
        raise ValueError("%r is not a valid Interface." % dotted_name)
    return klass


def getDottedName(iface):
    return f"{iface.__module__}.{iface.__name__}"


def _trim_doc_string(text):
    """
    Trims a doc string to make it format
    correctly with structured text.
    """
    text = text.strip().replace("\r\n", "\n")
    lines = text.split("\n")
    nlines = [lines[0]]
    if len(lines) > 1:
        min_indent = None
        for line in lines[1:]:
            indent = len(line) - len(line.lstrip())
            if indent < min_indent or min_indent is None:
                min_indent = indent
        for line in lines[1:]:
            nlines.append(line[min_indent:])
    return "\n".join(nlines)


def visitBaseInterfaces(iface, lst):
    bases = iface.getBases()
    for base in bases:
        if base in lst:
            return
        lst.append(base)
        visitBaseInterfaces(iface, lst)


@implementer(IInterfaceInformation)
class InterfaceInformation(BrowserView):
    @memoize
    def provides(self, dotted_name):
        iface = resolveInterface(dotted_name)
        return iface.providedBy(aq_base(self.context))

    @memoize
    def class_provides(self, dotted_name):
        iface = resolveInterface(dotted_name)
        return iface.providedBy(aq_base(self.context).__class__)

    @memoize
    def names_and_descriptions(self, dotted_name, all=0):
        """Returns a list of pairs (name, description) for a given
        interface"""
        iface = resolveInterface(dotted_name)
        data = iface.namesAndDescriptions(all=all)
        return [(name, desc.getDoc()) for name, desc in data]

    @memoize
    def get_interfaces(self):
        """Returns the list of interfaces which are implemented by the object"""
        return tuple(providedBy(aq_base(self.context)).flattened())

    def get_base_interface(self):
        """Returns all base interfaces of an object but no direct interfaces

        Base interfaces are the interfaces which are the super interfaces of
        the direct interfaces
        """
        ifaces = self.get_interfaces()
        bases = []
        for iface in ifaces:
            visitBaseInterfaces(iface, bases)
        return [biface for biface in bases if biface not in ifaces]

    def get_interface_informations(self, iface):
        """Gets all useful information from an iface

        * name
        * dotted name
        * trimmed doc string
        * base interfaces
        * methods with signature and trimmed doc string
        * attributes with trimemd doc string
        """
        bases = [base for base in iface.getBases()]

        attributes = []
        methods = []
        for name, desc in iface.namesAndDescriptions():
            if IMethod.providedBy(desc):
                methods.append(
                    {
                        "signature": desc.getSignatureString(),
                        "name": desc.getName(),
                        "doc": _trim_doc_string(desc.getDoc()),
                    }
                )
            else:
                attributes.append(
                    {
                        "name": desc.getName(),
                        "doc": _trim_doc_string(desc.getDoc()),
                    }
                )

        result = {
            "name": iface.getName(),
            "dotted_name": getDottedName(iface),
            "doc": _trim_doc_string(iface.getDoc()),
            "bases": bases,
            "base_names": [getDottedName(iface) for base in bases],
            "attributes": attributes,
            "methods": methods,
        }

        return result



from zope.deferredimport import deprecated


deprecated(
    "Import INextPreviousProvider from plone.app.dexterity.behaviors.nextprevious instead (will be removed in Plone 7)",
    INextPreviousProvider="plone.app.dexterity.behaviors.nextprevious:INextPreviousProvider",
)



from .interfaces import IContextState
from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.base.interfaces import INavigationRoot
from plone.base.interfaces import INonStructuralFolder
from plone.base.utils import pretty_title_or_id
from plone.memoize.view import memoize
from plone.portlets.interfaces import ILocalPortletAssignable
from plone.registry.interfaces import IRegistry
from Products.CMFCore.interfaces import IDynamicType
from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from Products.CMFDynamicViewFTI.interfaces import IBrowserDefault
from Products.Five.browser import BrowserView
from zope.component import getMultiAdapter
from zope.component import getUtility
from zope.component import queryAdapter
from zope.component import queryMultiAdapter
from zope.interface import implementer


BLACKLISTED_PROVIDERS = ("portal_workflow",)
BLACKLISTED_CATEGORIES = (
    "folder_buttons",
    "object_buttons",
)


@implementer(IContextState)
class ContextState(BrowserView):
    """Information about the state of the current context"""

    @memoize
    def current_page_url(self):
        url = self.current_base_url()
        query = self.request.get("QUERY_STRING", None)
        if query:
            url += "?" + query
        return url

    @memoize
    def current_base_url(self):
        return self.request.get(
            "ACTUAL_URL",
            self.request.get(
                "VIRTUAL_URL", self.request.get("URL", self.context.absolute_url())
            ),
        )

    @memoize
    def canonical_object(self):
        context = aq_inner(self.context)
        if self.is_default_page():
            return aq_parent(context)
        else:
            return context

    @memoize
    def canonical_object_url(self):
        return self.canonical_object().absolute_url()

    @memoize
    def view_url(self):
        """URL to use for viewing

        Files and Images get downloaded when they are directly
        called, instead of with /view appended.  We want to avoid that.
        """
        view_url = self.object_url()
        portal_type = getattr(aq_base(self.context), "portal_type", None)
        registry = getUtility(IRegistry)
        use_view_action = registry.get("plone.types_use_view_action_in_listings", [])
        if portal_type in use_view_action:
            view_url = view_url + "/view"
        return view_url

    @memoize
    def view_template_id(self):
        context = aq_inner(self.context)

        if IBrowserDefault.providedBy(context):
            browserDefault = context
        else:
            browserDefault = queryAdapter(context, IBrowserDefault)

        if browserDefault is not None:
            try:
                return browserDefault.getLayout()
            except AttributeError:
                # Might happen if FTI didn't migrate yet.
                pass

        action = self._lookupTypeActionTemplate("object/view")
        if not action:
            action = self._lookupTypeActionTemplate("folder/folderlisting")

        return action

    @memoize
    def is_view_template(self):
        current_url = self.current_base_url()
        canonical_url = self.canonical_object_url()
        object_url = self.object_url()

        if current_url.endswith("/"):
            current_url = current_url[:-1]

        if current_url == canonical_url or current_url == object_url:
            return True
        if not current_url.startswith(object_url):
            # Cut short.
            return False
        # Get the part of the current_url minus the object_url.
        last_part = current_url.split(object_url)[-1]
        if not last_part.startswith("/"):
            # Unexpected
            return False
        # Remove the slash from the front:
        last_part = last_part[1:]
        if last_part == "view":
            return True
        context = aq_inner(self.context)
        browserDefault = IBrowserDefault(context, None)
        if browserDefault is not None:
            fti = browserDefault.getTypeInfo()
            if fti.getMethodAliases().get(last_part) == "(Default)":
                return True

        template_id = self.view_template_id()
        if last_part == template_id:
            return True
        elif last_part == "@@%s" % template_id:
            return True

        return False

    @memoize
    def object_url(self):
        return aq_inner(self.context).absolute_url()

    @memoize
    def object_title(self):
        context = aq_inner(self.context)
        return pretty_title_or_id(context, context)

    @memoize
    def workflow_state(self):
        tool = getToolByName(self.context, "portal_workflow")
        return tool.getInfoFor(aq_inner(self.context), "review_state", None)

    def parent(self):
        return aq_parent(aq_inner(self.context))

    @memoize
    def folder(self):
        context = aq_inner(self.context)
        if self.is_structural_folder() and not self.is_default_page():
            return context
        else:
            return aq_parent(context)

    @memoize
    def is_folderish(self):
        return bool(
            getattr(aq_base(aq_inner(self.context)), "isPrincipiaFolderish", False)
        )

    @memoize
    def is_structural_folder(self):
        folderish = self.is_folderish()
        context = aq_inner(self.context)
        if not folderish:
            return False
        elif INonStructuralFolder.providedBy(context):
            return False
        else:
            return folderish

    @memoize
    def is_default_page(self):
        context = aq_inner(self.context)
        container = aq_parent(context)
        if not container:
            return False
        view = getMultiAdapter((container, self.request), name="default_page")
        return view.isDefaultPage(context)

    @memoize
    def is_portal_root(self):
        context = aq_inner(self.context)
        portal = getUtility(ISiteRoot)
        return aq_base(context) is aq_base(portal) or (
            self.is_default_page() and aq_base(aq_parent(context)) is aq_base(portal)
        )

    @memoize
    def is_navigation_root(self):
        context = aq_inner(self.context)
        return INavigationRoot.providedBy(context) or (
            self.is_default_page() and INavigationRoot.providedBy(aq_parent(context))
        )

    @memoize
    def is_editable(self):
        tool = getToolByName(self.context, "portal_membership")
        return bool(
            tool.checkPermission("Modify portal content", aq_inner(self.context))
        )

    @memoize
    def is_locked(self):
        # plone_lock_info is registered on marker interface ITTWLockable, since
        # not everything may want to parttake in its lock-stealing ways.
        lock_info = queryMultiAdapter(
            (self.context, self.request), name="plone_lock_info"
        )
        if lock_info is not None:
            return lock_info.is_locked_for_current_user()
        else:
            context = aq_inner(self.context)
            lockable = getattr(context.aq_explicit, "wl_isLocked", None) is not None
            return lockable and context.wl_isLocked()

    @memoize
    def is_toolbar_visible(self):
        tool = getToolByName(self.context, "portal_membership")
        return bool(tool.checkPermission("Show Toolbar", aq_inner(self.context)))

    @memoize
    def actions(self, category=None, max=-1):
        context = aq_inner(self.context)
        atool = getToolByName(context, "portal_actions")
        ttool = getToolByName(context, "portal_types")
        actions = []
        actions.extend(
            ttool.listActionInfos(
                object=context,
                category=category,
                max=max,
            )
        )
        actions.extend(
            atool.listActionInfos(
                object=context,
                categories=(category,),
                max=max,
            )
        )
        return actions

    def portlet_assignable(self):
        return ILocalPortletAssignable.providedBy(self.context)

    # Helper methods
    def _lookupTypeActionTemplate(self, actionId):
        context = aq_inner(self.context)
        if not IDynamicType.providedBy(context):
            # No type info available, so no actions to consult
            return None

        fti = context.getTypeInfo()
        actions = fti.listActionInfos(actionId, context, False, False, False)
        if not actions:
            # Action doesn't exist
            return None
        url = actions[0]["url"]
        if url.rstrip("/") == self.object_url().rstrip("/"):
            # (Default) action
            action = "(Default)"
        else:
            # XXX: This isn't quite right since it assumes the action starts
            # with ${object_url}
            action = url.split("/")[-1]

        # Try resolving method aliases because we need a real template_id here
        action = fti.queryMethodID(action, default=action, context=context)

        # Strip off leading /
        if action and action[0] == "/":
            action = action[1:]
        return action



from pathlib import Path
from setuptools import setup


version = "6.0.0a4"

long_description = (
    f"{Path('README.rst').read_text()}\n{Path('CHANGES.rst').read_text()}"
)

setup(
    name="plone.app.layout",
    version=version,
    description="Layout mechanisms for Plone",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    # Get more strings from
    # https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 6 - Mature",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 6.2",
        "Framework :: Plone :: Core",
        "Framework :: Zope :: 5",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
    ],
    keywords="plone layout viewlet",
    author="Plone Foundation",
    author_email="plone-developers@lists.sourceforge.net",
    url="https://pypi.org/project/plone.app.layout",
    license="GPL version 2",
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.10",
    install_requires=[
        "plone.app.content",
        "plone.app.dexterity",
        "plone.app.relationfield",
        "plone.app.uuid",
        "plone.app.viewletmanager >=1.2",
        "plone.base >=4.0.0a1",
        "plone.dexterity",
        "plone.formwidget.namedfile",
        "plone.i18n",
        "plone.memoize",
        "plone.portlets",
        "plone.protect",
        "Products.CMFEditions >=1.2.2",
        "Products.GenericSetup",
        "Products.statusmessages",
        "Products.ZCatalog",
        "Zope",
    ],
    extras_require=dict(
        test=[
            "plone.app.contenttypes[test]",
            "plone.app.relationfield",
            "plone.app.testing",
            "plone.browserlayer",
            "plone.dexterity",
            "plone.locking",
            "plone.testing",
            "z3c.relationfield",
            "zope.intid",
        ]
    ),
)



from zope.viewlet.interfaces import IViewletManager


class IHTTPHeaders(IViewletManager):
    """A viewlet manager that sits before <head> to set headers to the
    request.
    """


class IHtmlHead(IViewletManager):
    """A viewlet manager that sits in the <head> of the rendered page"""


class IHtmlHeadLinks(IViewletManager):
    """A viewlet manager that sits in the <head> and is responsible
    for semantic navigation links, the favicon, rss links and more
    provided by <link> tags.
    """


class IPortalTop(IViewletManager):
    """A viewlet manager that sits at the very top of the rendered page"""


class IMainNavigation(IViewletManager):
    """A viewlet manager that holds the main navigation viewlet"""


class IGlobalStatusMessage(IViewletManager):
    """A viewlet manager that is aside the content to render messages for
    the user.
    """


class IPortalHeader(IViewletManager):
    """A viewlet manager that sits right after the skip to content links"""


class IToolbar(IViewletManager):
    """A viewlet manager that sits above all content, normally used to hold
    the toolbar containing content views (tabs), associated actions and the
    personal tools.
    """


class IAboveContent(IViewletManager):
    """A viewlet manager that sits above the content area"""


class IAboveContentTitle(IViewletManager):
    """A viewlet manager that sits above the content title in view templates"""


class IDocumentActions(IViewletManager):
    """A viewlet manager that sits near the content heading"""


class IBelowContentTitle(IViewletManager):
    """A viewlet manager that sits below the content title in view templates"""


class IBelowContentDescription(IViewletManager):
    """A viewlet manager that sits below the content description in view templates"""


class IAboveContentBody(IViewletManager):
    """A viewlet manager that sits above the content body in view templates"""


class IBelowContentBody(IViewletManager):
    """A viewlet manager that sits below the content body in view templates"""


class IBelowContent(IViewletManager):
    """A viewlet manager that sits below the content area"""


class IPortalFooter(IViewletManager):
    """A viewlet manager that sits in the portal footer"""


class IScripts(IViewletManager):
    """A viewlet manager that stores the script tags"""



from plone.app.layout.testing import INTEGRATION_TESTING
from plone.base.utils import get_installer

import unittest


class TestSetup(unittest.TestCase):
    """Test plone.app.layout setup."""

    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

    def test_browserlayer(self):
        from plone.app.layout.interfaces import IPloneAppLayoutLayer
        from plone.browserlayer import utils

        self.assertIn(IPloneAppLayoutLayer, utils.registered_layers())


class TestUninstall(unittest.TestCase):
    layer = INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.installer = get_installer(self.portal, self.request)
        self.installer.uninstall_product("plone.app.layout")

    def test_browserlayer_removed(self):
        from plone.app.layout.interfaces import IPloneAppLayoutLayer
        from plone.browserlayer import utils

        self.assertNotIn(IPloneAppLayoutLayer, utils.registered_layers())
